@@ -2,19 +2,19 @@
  *  A Z-Machine
  *  Copyright (C) 2000 Andrew Hunter
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2.1 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /*
@@ -111,13 +111,16 @@ static inline void push(ZStack* stack, const ZWord word)
   
   if (stack->stack_size <= 0)
     {
+      int stack_offset = stack->stack_top - stack->stack;
+    
       stack->stack_total += 2048;
       if (!(stack->stack = realloc(stack->stack,
 				   stack->stack_total*sizeof(ZWord))))
 	{
 	  zmachine_fatal("Stack overflow");
 	}
-      stack->stack_size = 2048;
+      stack->stack_top = stack->stack + stack_offset;
+      stack->stack_size += 2048;
     }
 
 #ifdef DEBUG
@@ -509,7 +512,7 @@ static void draw_statusbar_123(ZStack* stack)
   stream_buffering(0);
   
   display_set_window(1); display_set_style(0); display_set_style(8);
-  display_set_colour(7, 0);
+  display_set_colour(rc_get_background(), rc_get_foreground());
 
   display_set_cursor(0, 0);
   display_erase_line(1);
@@ -530,7 +533,7 @@ static void draw_statusbar_123(ZStack* stack)
       display_printf("Score: %i  Moves: %i", score, moves);
     }
 
-  display_set_colour(0, 7); display_set_style(0);
+  display_set_colour(rc_get_foreground(), rc_get_background()); display_set_style(0);
   display_set_window(0);
   stream_buffering(1);
 }
@@ -1365,6 +1368,8 @@ static int newline_function(const int* remaining, int rem_len);
 void zcode_v6_initialise(void)
 {
   int x;
+  
+  machine.dinfo = display_get_info();
 
   for (x=0; x<8; x++)
     {
@@ -1391,8 +1396,6 @@ void zcode_v6_initialise(void)
   windows[0].wrapping = 1;
 
   v6_set_newline_function(newline_function);
-
-  machine.dinfo = display_get_info();
 }
 
 static int*  pending_text = NULL;
@@ -1610,10 +1613,10 @@ void zmachine_run(const int version,
   pc    = GetWord(machine.header, ZH_initpc);
   stack = &machine.stack;
 
+  machine.dinfo = display_get_info();
 #if defined(SUPPORT_VERSION_6)
   if (version == 6)
     {
-      machine.dinfo = display_get_info();
       zcode_v6_initialise();
     }
 #endif
