@@ -10,9 +10,10 @@
 
 #import "ZoomSkeinLayout.h"
 
+// Define this to use the 'new' skein colouring style
+#define SkeinDrawingStyleNew
+
 // Constants
-static const float itemWidth = 120.0; // Pixels
-static const float itemHeight = 96.0;
 static const float itemPadding = 56.0;
 
 static NSDictionary* itemTextAttributes = nil;
@@ -22,7 +23,11 @@ static NSDictionary* labelTextAttributes = nil;
 static NSString* ZoomNSShadowAttributeName = @"NSShadow";
 
 // Images
-static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
+static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation, *commentaryBadge;
+
+#ifdef SkeinDrawingStyleNew
+static NSImage* unchangedDark, *activeDark;
+#endif
 
 @implementation ZoomSkeinLayout
 
@@ -46,6 +51,35 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 	return img;
 }
 
++ (NSImage*) darkenImage: (NSImage*) image {
+	NSRect imgRect;
+	
+	imgRect.origin = NSMakePoint(0,0);
+	imgRect.size = [image size];
+
+	NSImage* highlighted = [[NSImage alloc] initWithSize: imgRect.size];
+	
+	[highlighted lockFocus];
+	
+	// Background
+	[[NSColor colorWithDeviceRed: 0.0
+						   green: 0.0
+							blue: 0.0
+						   alpha: 0.3] set];
+	NSRectFill(imgRect);
+	
+	// The item
+	[image drawAtPoint: NSMakePoint(0,0)
+			  fromRect: imgRect
+			 operation: NSCompositeDestinationAtop
+			  fraction: 1.0];
+	
+	[highlighted unlockFocus];
+	
+	// Release
+	return [highlighted autorelease];
+}
+
 + (void) initialize {
 	NSObject* labelShadow = nil;
 	
@@ -62,6 +96,12 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 	unchanged  = [[[self class] imageNamed: @"Skein-unchanged"] retain];
 	changed    = [[[self class] imageNamed: @"Skein-changed"] retain];
 	annotation = [[[self class] imageNamed: @"Skein-annotation"] retain];
+	commentaryBadge = [[[self class] imageNamed: @"SkeinDiffersBadge"] retain];
+	
+#ifdef SkeinDrawingStyleNew
+	unchangedDark = [[[self class] darkenImage: unchanged] retain];
+	activeDark = [[[self class] darkenImage: active] retain];
+#endif	
 	
 	itemTextAttributes = [[NSDictionary dictionaryWithObjectsAndKeys:
 		[NSFont systemFontOfSize: 10], NSFontAttributeName,
@@ -84,15 +124,6 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 	width = floorf(width);
 	
 	// Images must be 90x30
-	if (width == 90.0) {
-		[img drawAtPoint: pos
-				fromRect: NSMakeRect(0,0,90,30)
-			   operation: NSCompositeSourceOver
-				fraction: 1.0];
-		
-		return;
-	}
-	
 	if (width <= 0.0) width = 1.0;
 	
 	// Draw the middle bit
@@ -135,6 +166,10 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 	
 	if (self) {
 		rootItem = [item retain];
+
+		itemWidth = 120.0; // Pixels
+		itemHeight = 96.0;
+		packingStyle = IFSkeinPackTight;
 	}
 	
 	return self;
@@ -156,6 +191,15 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 
 // = Setting skein data =
 
+- (void) setItemWidth: (float) newItemWidth {
+	if (newItemWidth < 82.0) newItemWidth = 82.0;
+	itemWidth = newItemWidth;
+}
+
+- (void) setItemHeight: (float) newItemHeight {
+	itemHeight = newItemHeight;
+}
+
 - (void) setRootItem: (ZoomSkeinItem*) item {
 	if (rootItem) [rootItem release];
 	rootItem = [item retain];
@@ -166,8 +210,28 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 }
 
 - (void) setActiveItem: (ZoomSkeinItem*) item {
-	if (activeItem) [activeItem release];
+	if (activeItem) {
+		// If the new active item is not a child of the previous item, then unset the 'recently played' flag
+		if (![[activeItem children] containsObject: item]) {
+			ZoomSkeinItem* skeinItem = activeItem;
+			while (skeinItem != nil) {
+				ZoomSkeinLayoutItem* layoutItem = [itemForItem objectForKey: [NSValue valueWithPointer: skeinItem]];
+				[layoutItem setRecentlyPlayed: NO];
+				skeinItem = [skeinItem parent];
+			}
+		}
+		
+		[activeItem release];
+	}
 	activeItem = [item retain];
+	
+	// Mark everything upwards of the active item as played
+	ZoomSkeinItem* skeinItem = activeItem;
+	while (skeinItem != nil) {
+		ZoomSkeinLayoutItem* layoutItem = [itemForItem objectForKey: [NSValue valueWithPointer: skeinItem]];
+		[layoutItem setRecentlyPlayed: YES];
+		skeinItem = [skeinItem parent];
+	}
 }
 
 - (ZoomSkeinItem*) activeItem {
@@ -205,7 +269,8 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 	
 	while (currentItem != nil) {
 		// Store this item
-		[[itemForItem objectForKey: [NSValue valueWithPointer: currentItem]] setOnSkeinLine: YES];
+		ZoomSkeinLayoutItem* itemUpwards = [itemForItem objectForKey: [NSValue valueWithPointer: currentItem]];
+		[itemUpwards setOnSkeinLine: YES];
 		
 		// Up the tree
 		currentItem = [currentItem parent];
@@ -219,7 +284,10 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 		currentItem = [[[currentItem children] allObjects] objectAtIndex: 0];
 		
 		// Store this item
-		[[itemForItem objectForKey: [NSValue valueWithPointer: currentItem]] setOnSkeinLine: YES];
+		ZoomSkeinLayoutItem* itemUpwards = [itemForItem objectForKey: [NSValue valueWithPointer: currentItem]];
+		[itemUpwards setOnSkeinLine: YES];
+		[itemUpwards setRecentlyPlayed: NO];
+		// [[itemForItem objectForKey: [NSValue valueWithPointer: currentItem]] setOnSkeinLine: YES];
 	}
 }
 
@@ -235,8 +303,8 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 
 // = Performing layout =
 
-- (ZoomSkeinLayoutItem*) layoutSkeinItem: (ZoomSkeinItem*) item
-							   withLevel: (int) level {
+- (ZoomSkeinLayoutItem*) layoutSkeinItemLoose: (ZoomSkeinItem*) item
+									withLevel: (int) level {
 	if (item == nil) return nil;
 	
 	NSEnumerator* childEnum = [[item children] objectEnumerator];
@@ -250,8 +318,8 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 	
 	while (child = [childEnum nextObject]) {
 		// Layout the child item
-		childItem = [self layoutSkeinItem: child
-								withLevel: level+1];
+		childItem = [self layoutSkeinItemLoose: child
+									 withLevel: level+1];
 		
 		// Position it (first iteration: we center later)
 		position += lastWidth/2.0; // Add in halves: we're dealing with object centers
@@ -337,7 +405,7 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 		globalWidth = newPos;
 }
 
-- (void) layoutSkein {	
+- (void) layoutSkeinLoose {	
 	if (rootItem == nil) return;
 	
 	if (itemForItem) [itemForItem release];
@@ -354,8 +422,8 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 	}
 	levels = [[NSMutableArray alloc] init];
 	
-	tree = [[self layoutSkeinItem: rootItem
-						withLevel: 0] retain];
+	tree = [[self layoutSkeinItemLoose: rootItem
+							 withLevel: 0] retain];
 	
 	if (tree != nil) {
 		// Transform the 'relative' positions of all items into 'absolute' positions
@@ -502,21 +570,21 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 	ZoomSkeinLayoutItem* item;
 	
 	while (item = [levelEnum nextObject]) {
-		float itemWidth = [item width];
+		float thisItemWidth = [item width];
 		float itemPos = [item position] + globalOffset;
 		
 		// There's a +40 border either side of the item
-		itemWidth += 40.0;
+		thisItemWidth += 40.0;
 		
 		// Buttons require a minimum width
-		if (itemWidth < 72.0) {
-			itemWidth = 72.0;
+		if (thisItemWidth < 72.0) {
+			thisItemWidth = 72.0;
 		}
 		
 		// Item is centered
-		itemWidth /= 2.0;
+		thisItemWidth /= 2.0;
 		
-		if (point.x > (itemPos - itemWidth) && point.x < (itemPos + itemWidth)) {
+		if (point.x > (itemPos - thisItemWidth) && point.x < (itemPos + thisItemWidth)) {
 			// This is the item
 			return [item item];
 		}
@@ -571,11 +639,20 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 			float bgWidth = size.width;
 			//if (bgWidth < 90.0) bgWidth = 90.0;
 			
+#ifdef SkeinDrawingStyleNew
+			BOOL darken = [[skeinItem commentary] length] == 0;
+			
+			background = unchanged;
+			if (darken) background = unchangedDark;
+			if ([item recentlyPlayed] && !darken) background = active;
+			if ([item recentlyPlayed] && darken) background = activeDark;
+#else
 			if (![skeinItem played]) background = unplayed;
 			if ([skeinItem changed]) background = changed;
 			// if (skeinItem == activeItem) background = active;
 			if ([skeinItem parent] == activeItem) background = active;
 			// if (skeinItem == [self selectedItem]) background = selected;
+#endif
 			
 			[[self class] drawImage: background
 							atPoint: NSMakePoint(xpos - bgWidth/2.0, ypos-8 + (background==selected?2.0:0.0))
@@ -584,6 +661,19 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 			// Draw the item
 			[skeinItem drawCommandAtPosition: NSMakePoint(xpos - (size.width/2), ypos + (background==selected?2.0:0.0))];
 			
+			// Draw the 'commentary changed' badge if necessary
+			if ([skeinItem commentaryComparison] == ZoomSkeinDifferent) {
+				NSRect fromRect;
+				
+				fromRect.origin = NSMakePoint(0,0);
+				fromRect.size = [commentaryBadge size];
+				
+				[commentaryBadge drawAtPoint: NSMakePoint(xpos + bgWidth/2.0 + 4, ypos + 6)
+									fromRect: fromRect
+								   operation: NSCompositeSourceOver
+									fraction: 1.0];
+			}
+			
 			// Draw links to the children
 			[[NSColor blackColor] set];
 			NSEnumerator* childEnumerator = [[item children] objectEnumerator];
@@ -591,7 +681,11 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 			float startYPos = ypos + 10.0 + size.height;
 			float endYPos = ypos - 10.0 + itemHeight;
 			
+#ifdef SkeinDrawingStyleNew
+			NSColor* tempChildLink = [NSColor grayColor];
+#else
 			NSColor* tempChildLink = [NSColor blueColor];
+#endif
 			NSColor* permChildLink = [NSColor blackColor];
 			
 			ZoomSkeinLayoutItem* child;
@@ -601,20 +695,37 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 				
 				BOOL highlightLine = [child onSkeinLine];
 				
-				if ([[child item] temporary]) {
-					[tempChildLink set];
-				} else {
-					[permChildLink set];
-				}
-				
 				// Thicken the line if this is on the highlighted line
 				if (highlightLine) {
 					[NSBezierPath setDefaultLineWidth: 3.0];
 				}
 				
-				[NSBezierPath strokeLineFromPoint: NSMakePoint(xpos, startYPos)
-										  toPoint: NSMakePoint(childXPos, annotated?endYPos-18:endYPos)];
+				// Construct the line we're going to draw
+				NSBezierPath* line = [[NSBezierPath alloc] init];
+				[line moveToPoint: NSMakePoint(xpos, startYPos-8.0)];
+				[line lineToPoint: NSMakePoint(xpos, startYPos)];
+				[line lineToPoint: NSMakePoint(childXPos, annotated?endYPos-18:endYPos)];
+				[line lineToPoint: NSMakePoint(childXPos, annotated?endYPos-14:endYPos+10.0)];
 				
+				// Set the appropriate colour and dash pattern
+				if ([[child item] temporary]) {
+					float dashPattern[2];
+					
+					[tempChildLink set];
+					
+					dashPattern[0] = 4.0;
+					dashPattern[1] = 3.0;
+					[line setLineDash: dashPattern
+								count: 2
+								phase: 0.0];
+				} else {
+					[permChildLink set];
+				}
+				
+				// Draw the line
+				[line stroke];
+				[line release];
+								 
 				// Thin it out again afterwards
 				if (highlightLine) {
 					[NSBezierPath setDefaultLineWidth: 1.0];
@@ -623,12 +734,12 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 			
 			// Draw the annotation, if present
 			if ([[skeinItem annotation] length] > 0) {
-				float itemWidth = [self widthForItem: skeinItem];
+				float thisItemWidth = [self widthForItem: skeinItem];
 				float labelWidth = [skeinItem annotationSize].width;
 				
 				[[self class] drawImage: annotation
-								atPoint: NSMakePoint(xpos - itemWidth/2.0, ypos-30)
-							  withWidth: itemWidth];
+								atPoint: NSMakePoint(xpos - thisItemWidth/2.0, ypos-30)
+							  withWidth: thisItemWidth];
 				
 				[skeinItem drawAnnotationAtPosition: NSMakePoint(xpos - (labelWidth/2), ypos - 23)];
 			}
@@ -703,4 +814,148 @@ static NSImage* unplayed, *selected, *active, *unchanged, *changed, *annotation;
 	return res;
 }
 
+// = Alternative packing style(s) =
+
+- (void) setPackingStyle: (int) newPackingStyle {
+	packingStyle = newPackingStyle;
+}
+
+- (void) layoutSkein {
+	switch (packingStyle) {
+		case IFSkeinPackLoose:
+		default:
+			[self layoutSkeinLoose];
+			break;
+			
+		case IFSkeinPackTight:
+			[self layoutSkeinTight];
+			break;
+	}
+}
+
+- (void) layoutSkeinTight {
+	// 'Tight' packing style will always use horizontal space if it's available.
+	if (rootItem == nil) return;
+	
+	if (itemForItem) [itemForItem release];
+	itemForItem = [[NSMutableDictionary alloc] init];
+	
+	// Perform initial layout of the items
+	if (tree) {
+		[tree release];
+		tree = nil;
+	}
+	if (levels) {
+		[levels release];
+		levels = nil;
+	}
+	levels = [[NSMutableArray alloc] init];
+	
+	tree = [[self layoutSkeinItemTight: rootItem
+							 withLevel: 0] retain];
+	
+	if (tree != nil) {
+		// Transform the 'relative' positions of all items into 'absolute' positions
+		globalOffset = 0; globalWidth = 0;
+		[self fixPositions: tree
+				withOffset: 0];
+	}
+	
+	if (highlightedLineItem) [self updateHighlightDetails];
+}
+
+#define MaxTightDepth 8
+
+- (ZoomSkeinLayoutItem*) layoutSkeinItemTight: (ZoomSkeinItem*) item
+									withLevel: (int) level {
+	// Counterpart to layoutSkeinItemLoose that is slightly more intelligent about re-using vertical space
+	if (item == nil) return nil;
+	
+	NSEnumerator* childEnum = [[item children] objectEnumerator];
+	ZoomSkeinItem* child;
+	ZoomSkeinLayoutItem* lastItem = nil;
+	float position = 0.0;
+	float lastWidth = 0.0;
+	ZoomSkeinLayoutItem* childItem;
+	
+	NSMutableArray* children = [NSMutableArray array];
+	
+	while (child = [childEnum nextObject]) {
+		// Layout the child item
+		childItem = [self layoutSkeinItemTight: child
+									 withLevel: level+1];
+		
+		// Pick an effective item width
+		float effectiveWidthLeft = lastWidth;
+		float effectiveWidthRight = [childItem fullWidth];
+		
+		if (lastItem) {
+			int leftDepth = [lastItem depth];
+			int rightDepth = [childItem depth];
+			
+			if (leftDepth < rightDepth && leftDepth < MaxTightDepth) {
+				
+			} else if (rightDepth < leftDepth && rightDepth < MaxTightDepth) {
+				
+			}
+		}
+		
+		// Position it (first iteration: we center later)
+		position += effectiveWidthLeft/2.0; // Add in halves: we're dealing with object centers
+		
+		lastWidth = effectiveWidthRight;
+		position += lastWidth/2.0;
+		
+		[childItem setPosition: position];
+		
+		// Add to the list of children for this item
+		[children addObject: childItem];
+		lastItem = childItem;
+	}
+	
+	// Update position to be the total width
+	position += lastWidth/2.0;
+	
+	// Should only happen if there are no children
+	if (position == 0.0) position = itemWidth;
+	
+	// Center the children	
+	float center = position / 2.0;
+	
+	childEnum = [children objectEnumerator];
+	while (childItem = [childEnum nextObject]) {
+		[childItem setPosition: [childItem position] - center];
+	}
+	
+	// Adjust the width to fit the text, if required
+	float ourWidth = [item commandSize].width;
+	float labelWidth = [item annotationSize].width;
+	
+	if (labelWidth > ourWidth) ourWidth = labelWidth;
+	
+	if (position < (ourWidth + itemPadding)) position = ourWidth + itemPadding;
+	
+	// Return the result
+	ZoomSkeinLayoutItem* result = [[ZoomSkeinLayoutItem alloc] initWithItem: item
+																	  width: ourWidth
+																  fullWidth: position
+																	  level: level];
+	
+	[result setChildren: children];
+	
+	// Index this item
+	[itemForItem setObject: result
+					forKey: [NSValue valueWithPointer: item]];
+	
+	// Add to the 'levels' array, which contains which items to draw at which levels
+	while (level >= [levels count]) {
+		[levels addObject: [NSMutableArray array]];
+	}
+	
+	[[levels objectAtIndex: level] addObject: result];
+	[result release]; // Is retained in the levels array, so we don't autorelease (saves some time. Might cause bugs in the future though, so NOTE THIS)
+	
+	return result;
+}
+
 @end
