@@ -22,11 +22,13 @@
 @interface ZoomSkeinLayoutItem : NSObject {
 	ZoomSkeinItem* item;
 	BOOL		   onSkeinLine;
+	BOOL		   recentlyPlayed;
 	float          width;
 	float		   fullWidth;
 	float		   position;
 	NSArray*	   children;
-	int			   level;	
+	int			   level;
+	int			   depth;
 }
 
 // Initialisation
@@ -45,6 +47,8 @@
 - (NSArray*)	   children;
 - (int)			   level;
 - (BOOL)		   onSkeinLine;
+- (BOOL)		   recentlyPlayed;
+- (int)			   depth;
 
 - (void) setItem: (ZoomSkeinItem*) newItem;
 - (void) setWidth: (float) newWidth;
@@ -53,5 +57,8 @@
 - (void) setChildren: (NSArray*) newChildren;
 - (void) setLevel: (int) newLevel;
 - (void) setOnSkeinLine: (BOOL) onSkeinLine;
+- (void) setRecentlyPlayed: (BOOL) recentlyPlayed;
+
+- (NSArray*) itemsOnLevel: (int) level;
 
 @end
