@@ -12,8 +12,9 @@
 #include <Carbon/Carbon.h>
 
 // Constants
-static const float itemWidth = 120.0; // Pixels
-static const float itemHeight = 96.0;
+static const float defaultItemWidth = 120.0; // Pixels
+static const float defaultItemHeight = 96.0;
+static const float itemButtonBarWidth = 40.0;
 
 // Drawing info
 static NSDictionary* itemTextAttributes;
@@ -119,6 +120,9 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 		layout = [[ZoomSkeinLayout alloc] init];
 		[layout setRootItem: [skein rootItem]];
 		
+		itemWidth = defaultItemWidth;
+		itemHeight = defaultItemHeight;
+		
 		[self registerForDraggedTypes: [NSArray arrayWithObjects: ZoomSkeinItemPboardType, nil]];
     }
 	
@@ -128,15 +132,16 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 - (void) dealloc {
 	[skein release];
 	
-	if (trackingRects) [trackingRects release];
+	if (trackingRects)  [trackingRects release];
 
-	if (itemToEdit)    [itemToEdit release];
-	if (fieldScroller) [fieldScroller release];
-	if (fieldStorage)  [fieldStorage release];
+	if (itemToEdit)     [itemToEdit release];
+	if (fieldScroller)  [fieldScroller release];
+	if (fieldStorage)   [fieldStorage release];
 	
-	if (trackedItem)   [trackedItem release];
-	if (clickedItem)   [clickedItem release];
-	if (trackingItems) [trackingItems release];
+	if (trackedItem)    [trackedItem release];
+	if (clickedItem)    [clickedItem release];
+	if (trackingItems)  [trackingItems release];
+	if (mostRecentItem) [mostRecentItem release];
 
 	[layout release];
 	[[NSNotificationCenter defaultCenter] removeObserver: self];
@@ -217,7 +222,7 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 		// 
 		// Where A = Annotate, T = transcript, x = delete, + = add, L = lock
 		float w = bgWidth;
-		if (w < 32.0) w = 32.0;
+		if (w < itemButtonBarWidth) w = itemButtonBarWidth;
 		w += 40.0;
 		float left = xpos - w/2.0;
 		float right = xpos + w/2.0;
@@ -258,7 +263,7 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 			NSImage* lock = [trackedItem temporary]?unlocked:locked;
 			
 			[[self class] drawButton: lock
-							 atPoint: NSMakePoint(right, ypos + 18)
+							 atPoint: NSMakePoint(xpos - 8, ypos - 18)
 						 highlighted: activeButton==ZSVlockButton];
 		}
 	}
@@ -309,7 +314,8 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 	[self finishEditing: self];
 	[self skeinNeedsLayout];
 	
-	[self scrollToItem: [skein activeItem]];
+	[self scrollToItem: mostRecentItem];
+	[mostRecentItem release]; mostRecentItem = nil;
 }
 
 - (void) skeinNeedsLayout {
@@ -323,6 +329,23 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 	}
 }
 
+- (void) setItemWidth: (float) newItemWidth {
+	if (newItemWidth < 16.0) newItemWidth = 16.0;
+	if (newItemWidth == itemWidth) return;
+	itemWidth = newItemWidth;
+	
+	[self skeinNeedsLayout];
+}
+
+- (void) setItemHeight: (float) newItemHeight {
+	if (newItemHeight < 16.0) newItemHeight = 16.0;
+	if (newItemHeight == itemHeight) return;
+	itemHeight = newItemHeight;
+	
+	[self skeinNeedsLayout];
+	[self setNeedsDisplay: YES];
+}
+
 - (void) layoutSkein {
 	// Only actually layout if we're marked as needing it
 	if (!skeinNeedsLayout) return;
@@ -330,6 +353,8 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 	skeinNeedsLayout = NO;
 	
 	// Re-layout this skein
+	[layout setItemWidth: itemWidth];
+	[layout setItemHeight: itemHeight];
 	[layout setRootItem: [skein rootItem]];
 	[layout layoutSkein];
 	
@@ -350,9 +375,12 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 // = Affecting the display =
 
 - (void) scrollToItem: (ZoomSkeinItem*) item {
-	if (item == nil) return;
+	if (item == nil) item = [skein activeItem];
 	if ([self superview] == nil) return;
 	
+	[mostRecentItem release];
+	mostRecentItem = [item retain];
+	
 	if (skeinNeedsLayout) [self layoutSkein];
 	
 	ZoomSkeinLayoutItem* foundItem = [layout dataForItem: item];
@@ -368,9 +396,9 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 		xpos -= visRect.size.width / 2.0;
 		ypos -= visRect.size.height / 3.0;
 		
-		[self scrollPoint: NSMakePoint(xpos, ypos)];
+		[self scrollPoint: NSMakePoint(floorf(xpos), floorf(ypos))];
 	} else {
-		NSLog(@"ZoomSkeinView: Attempt to scroll to nonexistant item");
+		NSLog(@"ZoomSkeinView: Attempt to scroll to nonexistent item");
 	}
 }
 
@@ -546,6 +574,10 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 	return YES;
 }
 
+- (BOOL) acceptsFirstResponder {
+	return YES;
+}
+
 // = Mouse handling =
 
 - (void) mouseDown: (NSEvent*) event {
@@ -565,7 +597,7 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 	}
 	
 	if (clickedItem) [clickedItem release];
-	clickedItem = [trackedItem retain];
+	clickedItem = [realItem retain];
 	
 	if (trackedItem == nil) {
 		// We're dragging to move the view around
@@ -592,7 +624,7 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 		newVisRect.origin.x += dragOrigin.x - currentPos.x;
 		newVisRect.origin.y -= dragOrigin.y - currentPos.y;
 		
-		[self scrollRectToVisible: newVisRect];
+		[self scrollRectToVisible: NSIntegralRect(newVisRect)];
 	} else if (clickedItem != nil && (lastButton == ZSVmainItem)) {
 		// Drag this item. Default action is a copy action, but a move op is possible if command is held
 		// down.
@@ -695,6 +727,9 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 						if ([layout selectedItem] != trackedItem) {
 							// Change the selected item
 							[self setSelectedItem: trackedItem];
+							
+							// Edit soon
+							[self editSoon: trackedItem];
 						} else {
 							// Edit soon
 							[self editSoon: trackedItem];
@@ -725,10 +760,11 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 	NSSize size = [[item command] sizeWithAttributes: fontAttrs];
 
 	float w = size.width; //[[item objectForKey: ZSwidth] floatValue];
-	if (w < 32.0) w = 32.0;
+	if (w < itemButtonBarWidth) w = itemButtonBarWidth;
 	w += 40.0;
 	float left = -w/2.0;
 	float right = w/2.0;
+	float lozengeRight = size.width/2.0;
 	
 	// Correct for shadow
 	right -= 20.0;
@@ -744,9 +780,14 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 		if (offset.x > left+16.0 && offset.x < left+28.0) return ZSVtranscriptButton;
 		if (offset.x > right+2.0 && offset.x < right+14.0) return ZSVaddButton;
 		if (offset.x > right-12.0 && offset.x < right-0.0) return ZSVdeleteButton;
+		if (offset.x > -8 && offset.x < 8) return ZSVlockButton;
 	} else if (offset.y > 18.0 && offset.y < 30.0) {
 		// Lower row of buttons
-		if (offset.x > right+2.0 && offset.x < right+14.0) return ZSVlockButton;
+	} else if ([item commentaryComparison] == ZoomSkeinDifferent
+			   && offset.x > lozengeRight + 4.0 && offset.x < lozengeRight + 20.0
+			   && offset.y > 6.0 && offset.y < 22.0) {
+		// Comparison failed badge
+		return ZSVtranscriptButton;
 	} else if (offset.y > -2.0 && offset.y < 14.0) {
 		// Main item
 		return ZSVmainItem;
@@ -1248,4 +1289,297 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 	[self scrollToItem: item];
 }
 
+// = Context menu =
+
+- (NSMenu *)menuForEvent:(NSEvent *)event {
+	// Find which item that the mouse is over
+	NSPoint pointInView = [event locationInWindow];
+	pointInView = [self convertPoint: pointInView fromView: nil];
+	
+	contextItem = [layout itemAtPoint: pointInView];
+	
+	if (contextItem == nil) return nil;
+	
+	NSMenu* contextMenu = [[NSMenu alloc] init];
+	
+	// Add menu items for the standard actions
+	
+	[contextMenu addItemWithTitle: @"Play to Here"
+						   action: @selector(playToHere:)
+					keyEquivalent: @""];
+	
+	[contextMenu addItem: [NSMenuItem separatorItem]];
+
+	BOOL needSep = NO;
+	if ([contextItem parent] != nil) {
+		BOOL hasLabel = [[contextItem annotation] length] > 0;
+		needSep = YES;
+		[contextMenu addItemWithTitle: hasLabel?@"Edit Label":@"Add Label"
+							   action: @selector(addAnnotation:)
+						keyEquivalent: @""];
+	}
+	if ([delegate respondsToSelector: @selector(transcriptToPoint:)]) {
+		needSep = YES;
+		[contextMenu addItemWithTitle: @"Show in Transcript"
+							   action: @selector(showInTranscript:)
+						keyEquivalent: @""];
+	}
+	if ([contextItem parent] != nil) {
+		needSep = YES;
+		[contextMenu addItemWithTitle: [contextItem temporary]?@"Lock":@"Unlock"
+							   action: @selector(toggleLock:)
+						keyEquivalent: @""];
+		[contextMenu addItemWithTitle: [contextItem temporary]?@"Lock this Thread":@"Unlock this Branch"
+							   action: @selector(toggleLockBranch:)
+						keyEquivalent: @""];
+	}
+
+	if (needSep) [contextMenu addItem: [NSMenuItem separatorItem]];
+
+	if ([[contextItem children] count] > 0) {
+		[contextMenu addItemWithTitle: @"New Thread"
+							   action: @selector(addNewBranch:)
+						keyEquivalent: @""];
+	} else {
+		[contextMenu addItemWithTitle: @"Add New"
+							   action: @selector(addNewBranch:)
+						keyEquivalent: @""];
+	}
+
+	if ([contextItem parent] != nil) {
+		[contextMenu addItemWithTitle: @"Insert Knot"
+							   action: @selector(insertItem:)
+						keyEquivalent: @""];
+		if ([[contextItem children] count] > 0) {
+			[contextMenu addItemWithTitle: @"Delete"
+								   action: @selector(deleteOneItem:)
+							keyEquivalent: @""];
+			[contextMenu addItemWithTitle: @"Delete all Below"
+								   action: @selector(deleteItem:)
+							keyEquivalent: @""];
+		} else {
+			[contextMenu addItemWithTitle: @"Delete"
+								   action: @selector(deleteItem:)
+							keyEquivalent: @""];
+		}
+		[contextMenu addItemWithTitle: @"Delete all in Thread"
+							   action: @selector(deleteBranch:)
+						keyEquivalent: @""];
+	}
+	
+	[contextMenu addItem: [NSMenuItem separatorItem]];
+	[contextMenu addItemWithTitle: @"Save Transcript to Here..."
+						   action: @selector(saveTranscript:)
+					keyEquivalent: @""];
+	
+	// Return the menu
+	return [contextMenu autorelease];
+}
+
+// = Menu actions =
+
+- (IBAction) playToHere: (id) sender {
+	[self playToPoint: contextItem];
+}
+
+- (IBAction) showInTranscript: (id) sender {
+	[self transcriptButtonClicked: nil
+						 withItem: contextItem];
+}
+
+- (IBAction) addAnnotation: (id) sender {
+	[self editItemAnnotation: contextItem];
+}
+
+- (IBAction) toggleLock: (id) sender {
+	[contextItem setTemporary: ![contextItem temporary]];
+}
+
+- (IBAction) toggleLockBranch: (id) sender {
+	[contextItem setBranchTemporary: ![contextItem temporary]];
+}
+
+- (IBAction) addNewBranch: (id) sender {
+	// Add a new, blank item
+	ZoomSkeinItem* newItem = [contextItem addChild: [ZoomSkeinItem skeinItemWithCommand: @""]];
+	
+	// Lock it
+	[newItem setTemporary: NO];
+	
+	// Note the changes
+	[skein zoomSkeinChanged];	
+	[self skeinNeedsLayout];
+	
+	// Edit the item
+	[self scrollToItem: newItem];
+	[self editItem: newItem];
+}
+
+- (IBAction) insertItem: (id) sender {
+	// Get the parent item
+	ZoomSkeinItem* parent = [contextItem parent];
+	
+	// Remove any child items (these will become children of the new item)
+	NSArray* children = [[[[parent children] allObjects] copy] autorelease];
+	ZoomSkeinItem* child;
+	NSEnumerator* childEnum = [children objectEnumerator];;
+	
+	while (child = [childEnum nextObject]) {
+		[parent removeChild: child];
+	}
+	
+	// Add a new, blank item
+	ZoomSkeinItem* newItem = [parent addChild: [ZoomSkeinItem skeinItemWithCommand: @""]];
+	
+	// Add the child items back in again
+	childEnum = [children objectEnumerator];;
+	
+	while (child = [childEnum nextObject]) {
+		[newItem addChild: child];
+	}
+	
+	// Lock it
+	[newItem setTemporary: NO];
+	
+	// Note the changes
+	[skein zoomSkeinChanged];	
+	[self skeinNeedsLayout];
+	
+	// Edit the item
+	[self scrollToItem: newItem];
+	[self editItem: newItem];
+}
+
+- (BOOL) checkDelete: (ZoomSkeinItem*) skeinItem {
+	ZoomSkeinItem* itemParent = [skeinItem parent];
+	
+	if (itemParent == nil) return NO;
+	
+	ZoomSkeinItem* parent = [skein activeItem];
+	while (parent != nil) {
+		if (parent == skeinItem) {
+			if (![delegate respondsToSelector: @selector(cantDeleteActiveBranch)]) {
+				// Can't delete an item that's the parent of the active item
+				NSBeep();
+			} else {
+				[delegate cantDeleteActiveBranch];
+			}
+			return NO;
+		}
+		
+		parent = [parent parent];
+	}
+	
+	return YES;
+}
+
+- (IBAction) deleteItem: (id) sender {
+	if (![self checkDelete: contextItem]) return;
+	
+	ZoomSkeinItem* parent = [contextItem parent];
+	[parent removeChild: contextItem];
+	
+	// Force a layout of the skein
+	[self scrollToItem: parent];
+	[skein zoomSkeinChanged];
+	[self skeinNeedsLayout];
+}
+
+- (IBAction) deleteOneItem: (id) sender {
+	if (![self checkDelete: contextItem]) return;
+
+	// Remember the children of this item
+	NSArray* children = [[[[contextItem children] allObjects] copy] autorelease];
+
+	// Remove them from the tree
+	ZoomSkeinItem* child;
+	NSEnumerator* childEnum = [children objectEnumerator];;
+	
+	while (child = [childEnum nextObject]) {
+		[contextItem removeChild: child];
+	}
+	
+	// Remove this item (and any children)
+	ZoomSkeinItem* parent = [contextItem parent];
+	[parent removeChild: contextItem];
+	
+	// Add the children back to the parent item
+	childEnum = [children objectEnumerator];;
+	
+	while (child = [childEnum nextObject]) {
+		[parent addChild: child];
+	}
+	
+	// Force a layout of the skein
+	[skein zoomSkeinChanged];
+	[self skeinNeedsLayout];
+	
+	// Force a layout of the skein
+	[self scrollToItem: parent];
+	[skein zoomSkeinChanged];
+	[self skeinNeedsLayout];
+}
+
+- (IBAction) deleteBranch: (id) sender {
+	if (![self checkDelete: contextItem]) return;
+
+	// Find the top of this branch
+	ZoomSkeinItem* top = contextItem;
+	
+	while (top != nil && [top parent] != [skein rootItem] && [[[top parent] children] count] <= 1) {
+		top = [top parent];
+	}
+	
+	if (![self checkDelete: top]) return;
+	
+	// Remove the entire branch
+	ZoomSkeinItem* parent = [top parent];
+	[parent removeChild: top];
+	
+	// Force a layout of the skein
+	[self scrollToItem: parent];
+	[skein zoomSkeinChanged];
+	[self skeinNeedsLayout];
+}
+
+- (void) saveTranscript: (id) sender {
+	if ([self window] == nil) return;
+	
+	NSSavePanel* panel = [NSSavePanel savePanel];
+	[panel setRequiredFileType: @"txt"];
+	
+	NSString* directory = nil;
+	if (directory == nil) {
+		directory = [[NSUserDefaults standardUserDefaults] objectForKey: @"ZoomTranscriptPath"];
+	}
+	if (directory == nil) {
+		directory = NSHomeDirectory();
+	}
+	
+    [panel beginSheetForDirectory: directory
+                             file: nil
+                   modalForWindow: [self window]
+                    modalDelegate: self
+                   didEndSelector: @selector(saveTranscript:returnCode:contextInfo:) 
+                      contextInfo: [[skein transcriptToPoint: contextItem] retain]];
+}
+
+- (void) saveTranscript: (NSSavePanel *) panel 
+             returnCode: (int) returnCode 
+            contextInfo: (void*) contextInfo {
+	NSString* data = (NSString*)contextInfo;
+	[data autorelease];
+	
+	if (returnCode != NSOKButton) return;
+	
+	// Remember the directory we last saved in
+	[[NSUserDefaults standardUserDefaults] setObject: [panel directory]
+											  forKey: @"ZoomTranscriptPath"];
+	
+	// Save the data
+	NSData* stringData = [data dataUsingEncoding: NSUTF8StringEncoding];
+	[stringData writeToFile: [panel filename]
+				 atomically: YES];
+}
+
 @end
