@@ -43,33 +43,33 @@ static int cocoa_to_zscii(int theChar) {
 	// Deal with special keys (ie, convert to ZSCII)
 	switch (theChar) {
 		// Arrow keys
-		case NSUpArrowFunctionKey: key = 129; break;
-		case NSDownArrowFunctionKey: key = 130; break;
-		case NSLeftArrowFunctionKey: key = 131; break;
-		case NSRightArrowFunctionKey: key = 132; break;
+		case NSUpArrowFunctionKey: case 129: key = 129; break;
+		case NSDownArrowFunctionKey: case 130: key = 130; break;
+		case NSLeftArrowFunctionKey: case 131: key = 131; break;
+		case NSRightArrowFunctionKey: case 132: key = 132; break;
 			
 			// Delete/return
 		case 10: key = 13; break;
 		case NSDeleteFunctionKey: key = 8; break;
 			
 			// Function keys
-		case NSF1FunctionKey: key = 133; break;
-		case NSF2FunctionKey: key = 134; break;
-		case NSF3FunctionKey: key = 135; break;
-		case NSF4FunctionKey: key = 136; break;
-		case NSF5FunctionKey: key = 137; break;
-		case NSF6FunctionKey: key = 138; break;
-		case NSF7FunctionKey: key = 139; break;
-		case NSF8FunctionKey: key = 140; break;
-		case NSF9FunctionKey: key = 141; break;
-		case NSF10FunctionKey: key = 142; break;
-		case NSF11FunctionKey: key = 143; break;
-		case NSF12FunctionKey: key = 144; break;
+		case NSF1FunctionKey: case 133: key = 133; break;
+		case NSF2FunctionKey: case 134: key = 134; break;
+		case NSF3FunctionKey: case 135: key = 135; break;
+		case NSF4FunctionKey: case 136: key = 136; break;
+		case NSF5FunctionKey: case 137: key = 137; break;
+		case NSF6FunctionKey: case 138: key = 138; break;
+		case NSF7FunctionKey: case 139: key = 139; break;
+		case NSF8FunctionKey: case 140: key = 140; break;
+		case NSF9FunctionKey: case 141: key = 141; break;
+		case NSF10FunctionKey: case 142: key = 142; break;
+		case NSF11FunctionKey: case 143: key = 143; break;
+		case NSF12FunctionKey: case 144: key = 144; break;
 			
 			// Mouse buttons (we use fake function keys for this)
-		case NSF33FunctionKey: key = 252; break;
-		case NSF34FunctionKey: key = 254; break;
-		case NSF35FunctionKey: key = 253; break;
+		case NSF33FunctionKey: case 252: key = 252; break;
+		case NSF34FunctionKey: case 254: key = 254; break;
+		case NSF35FunctionKey: case 253: key = 253; break;
 			
 			// Numeric keypad
 		case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7':
@@ -138,7 +138,7 @@ ZDisplay* display_get_info(void) {
     dis.fixed_space   = 1;
     dis.sound_effects = 0;
     dis.timed_input   = 1;
-    dis.mouse         = 0;
+    dis.mouse         = 1;
 
     int xsize, ysize;
     int pxsize, pysize;
@@ -158,8 +158,8 @@ ZDisplay* display_get_info(void) {
     dis.font_width    = fontwidth;
     dis.font_height   = fontheight + 2.0;
     dis.pictures      = 1;
-    dis.fore          = 0; // Implement me: make configurable
-    dis.back          = 7; // Implement me: make configurable
+    dis.fore          = rc_get_foreground();
+    dis.back          = rc_get_background();
 	
 	NOTE(@"display_get_info");
 	
@@ -246,7 +246,7 @@ void display_erase_window(void) {
 void display_erase_line(int val) {
 	NOTE(@"display_erase_line");
 	
-    [[mainMachine buffer] eraseLineInWindow: [mainMachine windowNumber: zDisplayCurrentWindow]
+    [[mainMachine buffer] eraseLineInWindow: (NSObject<ZUpperWindow>*)[mainMachine windowNumber: zDisplayCurrentWindow]
                                   withStyle: zDisplayCurrentStyle];
 }
 
@@ -385,7 +385,7 @@ int display_readline(int* buf, int len, long int timeout) {
 	
 	if (prefix != nil && [prefix length] > 0) {
 		// Ask the display to backtrack input if possible
-		prefix = [display backtrackInputOver: prefix];
+		prefix = [[display backtrackInputOver: [prefix autorelease]] retain];
 	}
 
     NSDate* when;
@@ -431,8 +431,10 @@ int display_readline(int* buf, int len, long int timeout) {
     NSMutableString* inputBuffer = inputToDate==nil?[mainMachine inputBuffer]:[[inputToDate mutableCopy] autorelease];
 	
 	// Add the prefix, if any
-	[inputBuffer insertString: prefix 
-					  atIndex: 0];
+	if (prefix) {
+		[inputBuffer insertString: prefix 
+						  atIndex: 0];
+	}
 
 #ifdef DEBUG
 	NSLog(@"ZDisplay: display_readline = %@", inputBuffer);
@@ -652,8 +654,8 @@ void display_set_colour(int fore, int back) {
 	
     zDisplayCurrentStyle = [[zDisplayCurrentStyle autorelease] copy];
 
-    if (fore == -1) fore = 0;
-    if (back == -1) back = 7;
+    if (fore == -1) fore = rc_get_foreground();
+    if (back == -1) back = rc_get_background();
     
     if (fore < 16) {
         if (fore >= 0) {
@@ -679,7 +681,7 @@ void display_split(int lines, int window) {
 	NSLog(@"ZDisplay: display_split(%i, %i)", lines, window);
 #endif
 
-    [[mainMachine buffer] setWindow: [mainMachine windowNumber: window]
+    [[mainMachine buffer] setWindow: (NSObject<ZUpperWindow>*)[mainMachine windowNumber: window]
                           startLine: 0
                             endLine: lines];
 }
@@ -689,7 +691,7 @@ void display_join(int win1, int win2) {
 	NSLog(@"ZDisplay: display_join(%i, %i)", win1, win2);
 #endif
 	
-    [[mainMachine buffer] setWindow: [mainMachine windowNumber: win2]
+    [[mainMachine buffer] setWindow: (NSObject<ZUpperWindow>*)[mainMachine windowNumber: win2]
                           startLine: 0
                             endLine: 0];
 }
@@ -721,7 +723,7 @@ void display_set_cursor(int x, int y) {
 
     if (zDisplayCurrentWindow > 0) {
         [[mainMachine buffer] moveTo: NSMakePoint(x,y)
-                            inWindow: [mainMachine windowNumber: zDisplayCurrentWindow]];
+                            inWindow: (NSObject<ZUpperWindow>*)[mainMachine windowNumber: zDisplayCurrentWindow]];
     }
 }
 
