@@ -13,7 +13,7 @@
 
 #import "ZoomAppDelegate.h"
 
-#import "ifmetadata.h"
+#import "ifmetabase.h"
 
 // This deserves a note by itself: the autosave system is a bit weird. Strange bits are
 // handled by strange objects for strange reasons. This is because we have:
@@ -107,6 +107,9 @@
 	const unsigned char* bytes = [data bytes];
 	BOOL isForm = NO;
 	
+	// No valid game can be less than 40 bytes long (in fact, it must be longer but 40 bytes is needed for the ID, etc)
+	if ([data length] < 40) return NO;
+	
 	// See if this looks like a Blorb file (begins with 'FORM')
 	if (bytes[0] == 'F' && bytes[1] == 'O' && bytes[2] == 'R' && bytes[3] == 'M') isForm = YES;
 	
@@ -146,30 +149,27 @@
 		return NO;
 	}
 	
-	story = [[[NSApp delegate] userMetadata] findStory: storyId];
+	ZoomMetadata* userMetadata = [[NSApp delegate] userMetadata];
+	
+	story = [userMetadata containsStoryWithIdent: storyId]?[userMetadata findOrCreateStory: storyId]:nil;
 	
 	if (!story) {
 		// If there is no metadata, then make some up
 		story = [[NSApp delegate] findStory: storyId];
 		
 		if (story == nil) {
-			story = [[ZoomStory alloc] init];
-			[story setTitle: [[[self fileName] lastPathComponent] stringByDeletingPathExtension]];
+			story = [[ZoomStory defaultMetadataForFile: [self fileName]] retain];
 		} else {
 			[story retain];
 		}
 		
 		[story addID: storyId];
 		
-		[[[NSApp delegate] userMetadata] storeStory: [[story copy] autorelease]];
+		[[[NSApp delegate] userMetadata] copyStory: story];
 		[story release];
 		
-		story = [[[NSApp delegate] userMetadata] findStory: storyId];
-		if (story == nil) {
-			story = [[ZoomStory alloc] init];
-		} else {
-			[story retain];
-		}
+		story = [[[NSApp delegate] userMetadata] findOrCreateStory: storyId];
+		[story retain];
 	} else {
 		// If there is some metadata, then keep it around
 		[story retain];
@@ -194,7 +194,7 @@
 	} else if (resourceFilename != nil) {
 		// Resource file not found
 		[self addLoadingError: resources==nil?@"Failed to load resources: the resources set for this story could not be found":
-			@"Failed to load resources that were set for this story, but found alternatives in directory with story file"];
+			@"Failed to load resources that were set for this story, but found alternatives in the directory with story file"];
 		[story setObject: nil
 				  forKey: @"ResourceFilename"];
 	}
@@ -298,10 +298,15 @@
 		if (![self checkResourceFile: fileToCheck]) {
 			// foo.zlb
 			fileToCheck = [[resPath stringByAppendingPathComponent: resPrefix] stringByAppendingPathExtension: @"zlb"];
+
 			if (![self checkResourceFile: fileToCheck]) {
-				// resources.blb
-				fileToCheck = [resPath stringByAppendingPathComponent: @"resources.blb"];
-				[self checkResourceFile: fileToCheck];
+				// foo.zblorb
+				fileToCheck = [[resPath stringByAppendingPathComponent: resPrefix] stringByAppendingPathExtension: @"zblorb"];
+				if (![self checkResourceFile: fileToCheck]) {
+					// resources.blb
+					fileToCheck = [resPath stringByAppendingPathComponent: @"resources.blb"];
+					[self checkResourceFile: fileToCheck];
+				}
 			}
 		}
 	}
@@ -357,16 +362,9 @@
 				unsigned int checksum = (bytes[pos+12]<<8)|bytes[pos+13];
 				
 				// Set up the ZoomStoryID object for this savegame
-				struct IFMDIdent ident;
-				
-				ident.format = IFFormat_ZCode;
-				ident.usesMd5 = 0;
-				ident.dataFormat = IFFormat_ZCode;
-				ident.data.zcode.release = release;
-				ident.data.zcode.checksum = checksum;
-				memcpy(ident.data.zcode.serial, serial, 6);
-				
-				storyID = [[ZoomStoryID alloc] initWithIdent: &ident];
+				storyID = [[ZoomStoryID alloc] initWithZcodeRelease: release
+															 serial: serial
+														   checksum: checksum];
 				[storyID autorelease];
 			}
 		}
@@ -390,6 +388,22 @@
 		return NO;
 	}
 	
+	// If this isn't a single file, then try retrieving the ID from the Info.plist file (if it exists)
+	if (!isSingleFile) {
+		NSData* plist = [[[wrapper fileWrappers] objectForKey: @"Info.plist"] regularFileContents];
+		
+		if (plist != nil) {
+			NSDictionary* plistDict = [NSPropertyListSerialization propertyListFromData: plist
+																	   mutabilityOption: NSPropertyListImmutable
+																				 format: nil
+																	   errorDescription: nil];
+			NSString* idString  = [plistDict objectForKey: @"ZoomStoryId"];
+			if (idString != nil) {
+				storyID = [[[ZoomStoryID alloc] initWithIdString: idString] autorelease];
+			}
+		}
+	}
+	
 	// Get the game file for this save from the story organiser
 	NSString* gameFile = [[ZoomStoryOrganiser sharedStoryOrganiser] filenameForIdent: storyID];
 	[self findResourcesForFile: gameFile];
