@@ -19,6 +19,15 @@
 	BOOL shownOnce;
 	
 	NSRect oldWindowFrame;
+	
+	NSWindow* fullscreenWindow;							// Alternative window used for full-screen view
+	NSWindow* normalWindow;								// The usual window
+
+	float fadeTime;
+	float waitTime;
+	NSDate* fadeStart;
+	NSTimer* fadeTimer;
+	NSWindow* logoWindow;
 }
 
 - (IBAction) recordGameInfo: (id) sender;
@@ -27,5 +36,6 @@
 - (IBAction) playInFullScreen: (id) sender;
 
 - (ZoomView*) zoomView;
+- (void) showLogoWindow;
 
 @end
