@@ -11,10 +11,13 @@
 #import "ZoomClientController.h"
 #import "ZoomPreferenceWindow.h"
 #import "ZoomGameInfoController.h"
+#import "ZoomNotesController.h"
 #import "ZoomStoryOrganiser.h"
 #import "ZoomSkeinController.h"
 #import "ZoomConnector.h"
+#import "ZoomWindowThatCanBecomeKey.h"
 #import "ZoomAppDelegate.h"
+#import "ZoomClearView.h"
 
 @implementation ZoomClientController
 
@@ -34,7 +37,16 @@
 - (void) dealloc {
     if (zoomView) [zoomView setDelegate: nil];
     if (zoomView) [zoomView killTask];
-    
+	
+	if (fullscreenWindow) [fullscreenWindow release];
+	if (normalWindow) [normalWindow release];
+
+	if (fadeStart) [fadeStart release];
+	if (fadeTimer) {
+		[fadeTimer invalidate]; [fadeTimer release];
+	}
+	if (logoWindow) [logoWindow release];
+	
     [super dealloc];
 }
 
@@ -62,6 +74,7 @@
 	
 	// Add a skein view as an output receiver for the ZoomView
 	[zoomView addOutputReceiver: [[self document] skein]];
+	[self showLogoWindow];
 	
 	shownOnce = NO;
 }
@@ -92,6 +105,7 @@
 		}
 	}
 	
+	[self showLogoWindow];
 	shownOnce = YES;
 }
 
@@ -183,6 +197,17 @@
 	
 	[file addData: [[[[self document] skein] xmlData] dataUsingEncoding: NSUTF8StringEncoding]
 	  forFilename: @"Skein.skein"];
+	
+	// Add information about our story ID
+	[file addData: [NSPropertyListSerialization dataFromPropertyList: [NSDictionary dictionaryWithObjectsAndKeys: 
+		[[[self document] storyId] description], @"ZoomStoryId", nil]
+																									   format: NSPropertyListXMLFormat_v1_0
+																							 errorDescription: nil]
+													  forFilename: @"Info.plist"];
+}
+
+- (void) loadedSkeinData: (NSData*) skeinData {
+	[[[self document] skein] parseXmlData: skeinData];
 }
 
 - (NSString*) defaultSaveDirectory {
@@ -279,11 +304,16 @@
 	
 	if ([[ZoomGameInfoController sharedGameInfoController] infoOwner] == self) {
 		[self recordGameInfo: self];
+
 		[[ZoomGameInfoController sharedGameInfoController] setGameInfo: nil];
-		
 		[[ZoomGameInfoController sharedGameInfoController] setInfoOwner: nil];
 	}
-
+	
+	if ([[ZoomNotesController sharedNotesController] infoOwner] == self) {
+		[[ZoomNotesController sharedNotesController] setGameInfo: nil];
+		[[ZoomNotesController sharedNotesController] setInfoOwner: nil];
+	}
+	
 	if ([[ZoomSkeinController sharedSkeinController] skein] == [[self document] skein]) {
 		[[ZoomSkeinController sharedSkeinController] setSkein: nil];
 	}
@@ -292,7 +322,7 @@
 - (void) confirmFinish:(NSWindow *)sheet 
 			returnCode:(int)returnCode 
 		   contextInfo:(void *)contextInfo {
-	if (returnCode == NSAlertAlternateReturn) {
+	if (returnCode == NSAlertDefaultReturn) {
 		// Close the window
 		closeConfirmed = YES;
 		[[NSRunLoop currentRunLoop] performSelector: @selector(performClose:)
@@ -316,7 +346,7 @@
 		}
 		
 		NSBeginAlertSheet(@"Finish the game?",
-						  @"Keep playing", @"Finish", nil,
+						  @"Finish", @"Continue playing", nil,
 						  [self window], self,
 						  @selector(confirmFinish:returnCode:contextInfo:), nil,
 						  nil, msg);
@@ -372,6 +402,10 @@
 	
 	[[ZoomGameInfoController sharedGameInfoController] setGameInfo: [[self document] storyInfo]];
 	[[ZoomSkeinController sharedSkeinController] setSkein: [[self document] skein]];
+	[[zoomView textToSpeech] setSkein: [[self document] skein]];
+
+	[[ZoomNotesController sharedNotesController] setGameInfo: [[self document] storyInfo]];
+	[[ZoomNotesController sharedNotesController] setInfoOwner: self];
 }
 
 // = GameInfo updates =
@@ -416,6 +450,20 @@
 	[[[self document] storyInfo] setRating: [[ZoomGameInfoController sharedGameInfoController] rating]];
 }
 
+- (IBAction) infoResourceChanged: (id) sender {
+	ZoomStory* story = [[self document] storyInfo];
+	if (story == nil) return;
+	
+	// Update the resource path
+	[story setObject: [[ZoomGameInfoController sharedGameInfoController] resourceFilename]
+			  forKey: @"ResourceFilename"];
+	
+	// Perform organisation
+	if ([[ZoomPreferences globalPreferences] keepGamesOrganised]) {
+		[[ZoomStoryOrganiser sharedStoryOrganiser] organiseStory: story];
+	}
+}
+
 // = Various IB actions =
 
 - (IBAction) playInFullScreen: (id) sender {
@@ -427,18 +475,69 @@
 		[zoomView retain];
 		[zoomView removeFromSuperview];
 		
-		[[self window] setFrame: oldWindowFrame
-						display: YES
-						animate: YES];
-		[[self window] setShowsResizeIndicator: YES];
-		
 		[zoomView setScaleFactor: 1.0];
-		[zoomView setFrame: [[[self window] contentView] bounds]];
-		[[[self window] contentView] addSubview: zoomView];
+		[zoomView setFrame: [[normalWindow contentView] bounds]];
+		[[normalWindow contentView] addSubview: zoomView];
 		[zoomView release];
 		
+		// Swap windows back
+		if (normalWindow) {
+			[fullscreenWindow setDelegate: nil];
+			[fullscreenWindow setInitialFirstResponder: nil];
+			
+			[normalWindow setDelegate: self];
+			[normalWindow setWindowController: self];
+			[self setWindow: normalWindow];
+			[normalWindow setInitialFirstResponder: zoomView];
+			[normalWindow makeKeyAndOrderFront: self];
+
+			[fullscreenWindow orderOut: self];
+		}
+		
+		[self setWindowFrameAutosaveName: @"ZoomClientWindow"];
 		isFullscreen = NO;
 	} else {
+		// As of 10.4, we need to create a separate full-screen window (10.4 tries to be 'clever' with the window borders, which messes things up
+		if (!normalWindow) normalWindow = [[self window] retain];
+		if (!fullscreenWindow) {
+			fullscreenWindow = [[ZoomWindowThatCanBecomeKey alloc] initWithContentRect: [[[self window] contentView] bounds] 
+																				styleMask: NSBorderlessWindowMask
+																			   backing: NSBackingStoreBuffered
+																				 defer: YES];
+			
+			[fullscreenWindow setLevel: NSFloatingWindowLevel];
+			[fullscreenWindow setHidesOnDeactivate: YES];
+			[fullscreenWindow setReleasedWhenClosed: NO];
+			
+			if (![fullscreenWindow canBecomeKeyWindow]) {
+				[NSException raise: @"ZoomProgrammerIsASpoon"
+							format: @"For some reason, the full screen window won't accept key"];
+			}
+		}
+		
+		// Swap the displayed windows over
+		[self setWindowFrameAutosaveName: @""];
+		[fullscreenWindow setFrame: [normalWindow frame]
+						   display: NO];
+		[fullscreenWindow makeKeyAndOrderFront: self];
+		
+		[zoomView retain];
+		[zoomView removeFromSuperview];
+		[[fullscreenWindow contentView] addSubview: zoomView];
+		[zoomView release];
+		
+		[normalWindow setInitialFirstResponder: nil];
+		[normalWindow setDelegate: nil];
+
+		NSView* newFirstResponder = [zoomView textView];
+		if (newFirstResponder == nil) newFirstResponder = zoomView;
+		[fullscreenWindow setInitialFirstResponder: newFirstResponder];
+		[fullscreenWindow makeFirstResponder: newFirstResponder];
+		[fullscreenWindow setDelegate: self];
+
+		[fullscreenWindow setWindowController: self];
+		[self setWindow: fullscreenWindow];
+		
 		// Start being fullscreen
 		[[self window] makeKeyAndOrderFront: self];
 		oldWindowFrame = [[self window] frame];
@@ -449,17 +548,27 @@
 		[zoomView retain];
 		[zoomView removeFromSuperviewWithoutNeedingDisplay];
 		
-		// Resize the window
-		NSRect frame = [[[self window] screen] frame];
-		[[self window] setShowsResizeIndicator: NO];
-		frame = [NSWindow frameRectForContentRect: frame
-										styleMask: [[self window] styleMask]];
-		[[self window] setFrame: frame
-						display: YES
-						animate: YES];
-		
 		// Hide the menubar
 		[NSMenu setMenuBarVisible: NO];
+				
+		// Resize the window
+		NSRect frame = [[[self window] screen] frame];
+		if (![[NSApp delegate] leopard]) {
+			[[self window] setShowsResizeIndicator: NO];
+			frame = [NSWindow frameRectForContentRect: frame
+											styleMask: NSBorderlessWindowMask];
+			[[self window] setFrame: frame
+							display: YES
+							animate: YES];			
+			[normalWindow orderOut: self];
+		} else {
+			[fullscreenWindow setOpaque: NO];
+			[fullscreenWindow setBackgroundColor: [NSColor clearColor]];
+			[[self window] setContentView: [[[ZoomClearView alloc] init] autorelease]];
+			[[self window] setFrame: frame
+							display: YES
+							animate: NO];
+		}
 		
 		// Resize, reposition the zoomView
 		NSRect newZoomViewFrame = [[[self window] contentView] bounds];
@@ -476,10 +585,168 @@
 		[[[self window] contentView] addSubview: zoomView];
 		[zoomView release];
 		
+		// Perform an animation in Leopard
+		if ([[NSApp delegate] leopard]) {
+			[[[NSApp delegate] leopard] fullScreenView: zoomView
+											 fromFrame: oldWindowFrame
+											   toFrame: frame];			
+		}
+		
 		isFullscreen = YES;
 	}
 }
 
+// = Showing a logo =
+
+- (NSImage*) resizeLogo: (NSImage*) input {
+	NSSize oldSize = [input size];
+	NSImage* result = input;
+	
+	if (oldSize.width > 256 || oldSize.height > 256) {
+		float scaleFactor;
+		
+		if (oldSize.width > oldSize.height) {
+			scaleFactor = 256/oldSize.width;
+		} else {
+			scaleFactor = 256/oldSize.height;
+		}
+		
+		NSSize newSize = NSMakeSize(scaleFactor * oldSize.width, scaleFactor * oldSize.height);
+		
+		result = [[[NSImage alloc] initWithSize: newSize] autorelease];
+		[result lockFocus];
+		[[NSGraphicsContext currentContext] setImageInterpolation: NSImageInterpolationHigh];
+		
+		[input drawInRect: NSMakeRect(0,0, newSize.width, newSize.height)
+				 fromRect: NSMakeRect(0,0, oldSize.width, oldSize.height)
+				operation: NSCompositeSourceOver
+				 fraction: 1.0];
+		[result unlockFocus];
+	}
+	
+	return result;
+}
+
+- (NSImage*) logo {
+	NSImage* result = [ZoomStoryOrganiser frontispieceForFile: [[self document] fileName]];
+	if (result == nil) return nil;
+	
+	return [self resizeLogo: result];
+}
+
+- (void) positionLogoWindow {
+	// Position relative to the window
+	NSRect frame = [[[self window] contentView] convertRect: [[[self window] contentView] bounds] toView: nil];
+	NSRect windowFrame = [[self window] frame];
+	
+	// Position on screen
+	frame.origin.x += windowFrame.origin.x;
+	frame.origin.y += windowFrame.origin.y;
+	
+	// Position the logo window
+	[logoWindow setFrame: frame
+				 display: YES];
+}
+
+- (void) showLogoWindow {
+	// Fading the logo out like this stops it from flickering
+	waitTime = 1.0;
+	fadeTime = 0.5;
+	NSImage* logo = [self logo];
+	
+	if (logo == nil) return;
+	if (logoWindow) return;
+	if (fadeTimer) return;
+	
+	// Don't show this if this view is not on the screen
+	if (![[ZoomPreferences globalPreferences] showCoverPicture]) return;
+	if ([self window] == nil) return;
+	if (![[self window] isVisible]) return;
+	
+	// Create the window
+	logoWindow = [[NSWindow alloc] initWithContentRect: [[[self window] contentView] frame]				// Gets the size, we position later
+											 styleMask: NSBorderlessWindowMask
+											   backing: NSBackingStoreBuffered
+												 defer: YES];
+	[logoWindow setOpaque: NO];
+	[logoWindow setBackgroundColor: [NSColor clearColor]];
+	
+	// Create the image view that goes inside
+	NSImageView* fadeContents = [[NSImageView alloc] initWithFrame: [[logoWindow contentView] frame]];
+	
+	[fadeContents setImage: logo];
+	[logoWindow setContentView: [fadeContents autorelease]];
+	
+	fadeTimer = [NSTimer timerWithTimeInterval: waitTime
+										target: self
+									  selector: @selector(startToFadeLogo)
+									  userInfo: nil
+									   repeats: NO];
+	[[NSRunLoop currentRunLoop] addTimer: fadeTimer
+								 forMode: NSDefaultRunLoopMode];
+	
+	// Position the window correctly
+	[self positionLogoWindow];
+	
+	// Show the window
+	[logoWindow orderFront: self];
+	[[self window] addChildWindow: logoWindow
+						  ordered: NSWindowAbove];
+}
+
+- (void) startToFadeLogo {
+	fadeTimer = nil;
+	
+	fadeTimer = [NSTimer timerWithTimeInterval: 0.01
+										target: self
+									  selector: @selector(fadeLogo)
+									  userInfo: nil
+									   repeats: YES];
+	[[NSRunLoop currentRunLoop] addTimer: fadeTimer
+								 forMode: NSDefaultRunLoopMode];
+	
+	[fadeStart release];
+	fadeStart = [[NSDate date] retain];
+}
+
+- (void) fadeLogo {
+	float timePassed = [[NSDate date] timeIntervalSinceDate: fadeStart];
+	float fadeAmount = timePassed/fadeTime;
+	
+	if (fadeAmount < 0 || fadeAmount > 1) {
+		// Finished fading: get rid of the window + the timer
+		[fadeTimer invalidate];
+		fadeTimer = nil;
+		
+		[[logoWindow parentWindow] removeChildWindow: logoWindow];
+		[logoWindow release];
+		logoWindow = nil;
+		
+		[fadeStart release];
+		fadeStart = nil;
+	} else {
+		fadeAmount = -2.0*fadeAmount*fadeAmount*fadeAmount + 3.0*fadeAmount*fadeAmount;
+		
+		[logoWindow setAlphaValue: 1.0 - fadeAmount];
+	}
+}
+
+// = Interacting with the skein =
+
+- (void) restartGame {
+	 // Will force a restart
+	 [[self zoomView] runNewServer: nil];
+}
+
+- (void) playToPoint: (ZoomSkeinItem*) point
+		   fromPoint: (ZoomSkeinItem*) fromPoint {
+	 id inputSource = [ZoomSkein inputSourceFromSkeinItem: fromPoint
+												   toItem: point];
+	 
+	 
+	 [[self zoomView] setInputSource: inputSource];
+}
+
 // = Window title =
 
 - (NSString *)windowTitleForDocumentDisplayName:(NSString *)displayName {
@@ -494,4 +761,23 @@
 	return zoomView;
 }
 
+// = Text to speech =
+
+- (IBAction) stopSpeakingMove: (id) sender {
+	[[zoomView textToSpeech] beQuiet];
+}
+
+- (IBAction) speakMostRecent: (id) sender {
+	[[zoomView textToSpeech] resetMoves];
+	[[zoomView textToSpeech] speakLastText];
+}
+
+- (IBAction) speakNext: (id) sender {
+	[[zoomView textToSpeech] speakNextMove];
+}
+
+- (IBAction) speakPrevious: (id) sender {
+	[[zoomView textToSpeech] speakPreviousMove];
+}
+
 @end
