@@ -16,6 +16,7 @@
 	
 	BOOL rearranging;
 	BOOL reiterate;
+	int iterationCount;
 }
 
 - (void) addSubview: (NSView*) subview
@@ -23,6 +24,9 @@
 - (void) startRearranging;
 - (void) finishRearranging;
 - (void) rearrangeSubviews;
+- (void) removeAllSubviews;
+- (void) setSubview: (NSView*) subview
+		   isHidden: (BOOL) isHidden;
 - (void) subviewFrameChanged: (NSNotification*) not;
 
 @end
