@@ -25,6 +25,7 @@ static NSString* autosaveGames		= @"autosaveGames";
 
 static NSString* gameTitle			= @"GameTitle";
 static NSString* interpreter		= @"Interpreter";
+static NSString* glulxInterpreter	= @"GlulxInterpreter";
 static NSString* revision			= @"Revision";
 
 static NSString* fonts				= @"Fonts";
@@ -186,6 +187,8 @@ static NSArray* DefaultColours(void) {
 				  forKey: interpreter];
 		[prefs setObject: [NSNumber numberWithInt: 'Z']
 				  forKey: revision];
+		[prefs setObject: [NSNumber numberWithInt: GlulxGit]
+				  forKey: glulxInterpreter];
 		
 		[prefs setObject: DefaultFonts()
 				  forKey: fonts];
@@ -344,6 +347,19 @@ static NSArray* DefaultColours(void) {
 	return result;
 }
 
+- (enum GlulxInterpreter) glulxInterpreter {
+	[prefLock lock];
+	NSNumber* glulxInterpreterNum = (NSNumber*)[prefs objectForKey: glulxInterpreter];
+	
+	enum GlulxInterpreter result;
+	if (glulxInterpreterNum)	result = [glulxInterpreterNum intValue];
+	else						result = GlulxGit;
+	
+	[prefLock unlock];
+	
+	return result;
+}
+
 - (unsigned char) revision {
 	[prefLock lock];
 	unsigned char result = [[prefs objectForKey: revision] intValue];
@@ -531,6 +547,12 @@ static NSArray* DefaultColours(void) {
 	[self preferencesHaveChanged];
 }
 
+- (void) setGlulxInterpreter: (enum GlulxInterpreter) value {
+	[prefs setObject: [NSNumber numberWithInt: value]
+			  forKey: glulxInterpreter];
+	[self preferencesHaveChanged];
+}
+
 - (void) setInterpreter: (int) inter {
 	[prefs setObject: [NSNumber numberWithInt: inter]
 			  forKey: interpreter];
