@@ -49,7 +49,21 @@ static NSImage* saveBackground;
 	return self;
 }
 
+- (id) initWithPreviewStrings: (NSArray*) prev
+					 filename: (NSString*) file {
+	self = [self init];
+	
+	if (self) {
+		previewLines = [prev retain];
+		filename = [file copy];
+		highlighted = NO;
+	}
+	
+	return self;
+}
+
 - (void) dealloc {
+	if (previewLines) [previewLines release];
 	if (preview) [preview release];
 	if (filename) [filename release];
 	
@@ -110,8 +124,14 @@ static NSImage* saveBackground;
 	float ypos = 4;
 	int lines = 0;
 	
-	NSArray* upperLines = [preview lines];
-	NSAttributedString* thisLine;
+	NSArray* upperLines = nil;
+	id thisLine;
+	
+	if (preview) {
+		upperLines = [preview lines];
+	} else if (previewLines) {
+		upperLines = previewLines;
+	}
 	
 	NSEnumerator* lineEnum = [upperLines objectEnumerator];
 	
@@ -122,11 +142,18 @@ static NSImage* saveBackground;
 		unichar* newString = NULL;
 		int newLen = 0;
 
+		NSString* lineString;
+		if ([thisLine isKindOfClass: [NSString class]]) {
+			lineString = thisLine;
+		} else {
+			lineString = [thisLine string];
+		}
+		
 		for (x=0; x<[thisLine length]; x++) {
-			unichar chr = [[thisLine string] characterAtIndex: x];
+			unichar chr = [lineString characterAtIndex: x];
 			
 			if (chr == 32) {
-				while (x<[thisLine length]-1 && [[thisLine string] characterAtIndex: x+1] == 32) {
+				while (x<[lineString length]-1 && [lineString characterAtIndex: x+1] == 32) {
 					x++;
 				}
 			}
@@ -196,6 +223,9 @@ static NSImage* saveBackground;
 	}
 }
 
+- (void) mouseDown: (NSEvent*) event {
+}
+
 - (void) mouseUp: (NSEvent*) event {
 	ZoomSavePreviewView* superview = (ZoomSavePreviewView*)[self superview];
 	
@@ -219,7 +249,7 @@ static NSImage* saveBackground;
 - (IBAction) deleteSavegame: (id) sender {
 	// Display a confirmation dialog
 	NSBeginAlertSheet(@"Are you sure?", 
-					  @"Keep", @"Delete", nil, nil, self, @selector(confirmDelete:returnCode:contextInfo:), 
+					  @"Delete", @"Keep", nil, nil, self, @selector(confirmDelete:returnCode:contextInfo:), 
 					  nil, nil,
 					  @"Are you sure you want to delete this saved game?");
 	
@@ -228,7 +258,7 @@ static NSImage* saveBackground;
 
 - (void) confirmDelete:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {
 	// Check the user confirmed the delete
-	if (returnCode != NSAlertAlternateReturn) return;
+	if (returnCode != NSAlertDefaultReturn) return;
 	
 	// Ensure that this is a genuine savegame
 	BOOL genuine = YES;
@@ -237,7 +267,8 @@ static NSImage* saveBackground;
 	
 	NSString* saveDir = [[self filename] stringByDeletingLastPathComponent];
 	
-	if (![[[saveDir pathExtension] lowercaseString] isEqualToString: @"zoomsave"]) {
+	if (![[[saveDir pathExtension] lowercaseString] isEqualToString: @"zoomsave"]
+		&& ![[[saveDir pathExtension] lowercaseString] isEqualToString: @"glksave"]) {
 		genuine = NO; reason = reason?reason:[NSString stringWithFormat: @"File has the wrong extension (%@)", [saveDir pathExtension]];
 	}
 	if (![[NSFileManager defaultManager] fileExistsAtPath: saveDir
