@@ -21,8 +21,11 @@
 	
 	float scaleFactor;
 	
+	NSSize lastFixedSize;
 	NSSize lastTileSize;
 	int lastUpperSize;
+	
+	BOOL useDivider;
 }
 
 - (id) initWithFrame: (NSRect) frame
@@ -32,4 +35,6 @@
 - (void) updateUpperWindows;
 - (ZoomUpperWindowView*) upperWindowView;
 
+- (BOOL) setUseUpperDivider: (BOOL) useDivider;
+
 @end
