@@ -22,6 +22,7 @@
 		
 		lastTileSize = NSMakeSize(-1,-1);
 		lastUpperSize = -1;
+		useDivider = YES;
     }
     return self;
 }
@@ -54,12 +55,12 @@
 	}
 
 	int upperHeight  = [zoomView upperWindowSize];
-
-	if (!NSEqualSizes(lastTileSize, thisTileSize) || lastUpperSize != upperHeight) {
+	NSSize fixedSize = [@"M" sizeWithAttributes:
+		[NSDictionary dictionaryWithObjectsAndKeys:
+			[zoomView fontWithStyle:ZFixedStyle], NSFontAttributeName, nil]];	
+	
+	if (!NSEqualSizes(lastTileSize, thisTileSize) || lastUpperSize != upperHeight || !NSEqualSizes(lastFixedSize, fixedSize)) {
 		// Move the content view to accomodate the upper window
-		NSSize fixedSize = [@"M" sizeWithAttributes:
-			[NSDictionary dictionaryWithObjectsAndKeys:
-				[zoomView fontWithStyle:ZFixedStyle], NSFontAttributeName, nil]];
 
 		double upperMargin = (upperHeight * fixedSize.height) / scaleFactor;
 
@@ -84,6 +85,7 @@
 		upperFrame.size.height = upperMargin;
 
 		double sepHeight = [upperDivider frame].size.height;
+		if (!useDivider) sepHeight = 0;
 
 		// Actually resize the contentView
 		contentFrame.origin.y    += sepHeight;
@@ -92,13 +94,17 @@
 		[contentView setFrame: contentFrame];
 
 		// The upper/lower view seperator
-		sepFrame = [upperDivider frame];
-		sepFrame = contentFrame;
-		sepFrame.origin.y -= sepHeight;
-		sepFrame.size.height = sepHeight;
-		[upperDivider setFrame: sepFrame];
-		if ([upperDivider superview] == nil) [self addSubview: upperDivider];
-		[upperDivider setNeedsDisplay: YES];
+		if (useDivider) {
+			sepFrame = [upperDivider frame];
+			sepFrame = contentFrame;
+			sepFrame.origin.y -= sepHeight;
+			sepFrame.size.height = sepHeight;
+			[upperDivider setFrame: sepFrame];
+			if ([upperDivider superview] == nil) [self addSubview: upperDivider];
+			[upperDivider setNeedsDisplay: YES];
+		} else {
+			if ([upperDivider superview] != nil) [upperDivider removeFromSuperview];
+		}
 
 		// The upper window view
 		[zoomView setUpperBuffer: (upperMargin*scaleFactor) + sepHeight];
@@ -127,6 +133,7 @@
 	// Update the cache of how we were last resized
 	lastTileSize = [self bounds].size;
 	lastUpperSize = upperHeight;
+	lastFixedSize = fixedSize;
 }
 
 - (void) updateUpperWindows {
@@ -143,4 +150,11 @@
 	return upperView;
 }
 
+- (BOOL) setUseUpperDivider: (BOOL) newUseDivider {
+	useDivider = newUseDivider;
+	[self tile];
+	return YES;
+}
+
+
 @end
