@@ -15,6 +15,8 @@
 #import "ZoomStoryOrganiser.h"
 #import "ZoomAppDelegate.h"
 #import "ZoomPreferences.h"
+#import "ZoomPlugInManager.h"
+#import "ZoomPlugIn.h"
 
 NSString* ZoomStoryOrganiserChangedNotification = @"ZoomStoryOrganiserChangedNotification";
 NSString* ZoomStoryOrganiserProgressNotification = @"ZoomStoryOrganiserProgressNotification";
@@ -26,6 +28,64 @@ static NSString* ZoomIdentityFilename = @".zoomIdentity";
 
 @implementation ZoomStoryOrganiser
 
+// = Shared functions =
+
++ (NSImage*) frontispieceForBlorb: (ZoomBlorbFile*) decodedFile {
+	int coverPictureNumber = -1;
+	
+	// Try to retrieve the frontispiece tag (overrides metadata if present)
+	NSData* front = [decodedFile dataForChunkWithType: @"Fspc"];
+	if (front != nil && [front length] >= 4) {
+		const unsigned char* fpc = [front bytes];
+		
+		coverPictureNumber = (((int)fpc[0])<<24)|(((int)fpc[1])<<16)|(((int)fpc[2])<<8)|(((int)fpc[3])<<0);
+	}
+	
+	if (coverPictureNumber >= 0) {			
+		// Attempt to retrieve the cover picture image
+		if (decodedFile != nil) {
+			NSData* coverPictureData = [decodedFile imageDataWithNumber: coverPictureNumber];
+			
+			if (coverPictureData) {
+				NSImage* coverPicture = [[[NSImage alloc] initWithData: coverPictureData] autorelease];
+				
+				// Sometimes the image size and pixel size do not match up
+				NSImageRep* coverRep = [[coverPicture representations] objectAtIndex: 0];
+				NSSize pixSize = NSMakeSize([coverRep pixelsWide], [coverRep pixelsHigh]);
+				
+				if (!NSEqualSizes(pixSize, [coverPicture size])) {
+					[coverPicture setScalesWhenResized: YES];
+					[coverPicture setSize: pixSize];
+				}
+				
+				if (coverPicture != nil) {
+					return coverPicture;
+				}
+			}
+		}
+	}
+	
+	return nil;
+}
+
++ (NSImage*) frontispieceForFile: (NSString*) filename {
+	// First see if a plugin can provide the image...
+	ZoomPlugIn* plugin = [[ZoomPlugInManager sharedPlugInManager] instanceForFile: filename];
+	NSImage* res = nil;
+	if (plugin != nil) {
+		res = [plugin coverImage];
+		if (res != nil) return res;
+	} else {
+		// Then try using the standard blorb decoder
+		ZoomBlorbFile* decodedFile = [[[ZoomBlorbFile alloc] initWithContentsOfFile: filename] autorelease];
+		
+		if (decodedFile != nil) res = [self frontispieceForBlorb: decodedFile];
+		if (res != nil) return res;
+	}
+
+	return nil;
+}
+
 // = Internal functions =
 
 - (NSDictionary*) dictionary {
@@ -55,6 +115,11 @@ static NSString* ZoomIdentityFilename = @".zoomIdentity";
 											  forKey:extraDefaultsName];
 }
 
+- (out bycopy ZoomStoryID*) idForFile: (in bycopy NSString*) filename {
+	if (![[NSFileManager defaultManager] fileExistsAtPath: filename]) return nil;
+	return [ZoomStoryID idForFile: filename];
+}
+
 - (void) preferenceThread: (NSDictionary*) threadDictionary {
 	NSAutoreleasePool* p = [[NSAutoreleasePool alloc] init];
 	NSDictionary* prefs = [threadDictionary objectForKey: @"preferences"];
@@ -68,19 +133,27 @@ static NSString* ZoomIdentityFilename = @".zoomIdentity";
 	subThread = [[NSConnection allocWithZone: [self zone]]
         initWithReceivePort: port2
                    sendPort: port1];
-	[subThread setRootObject: self];
 	
 	// Notify the main thread that things are happening
-	[(ZoomStoryOrganiser*)[subThread rootProxy] startedActing];
+	ZoomStoryOrganiser* rootProxy = (ZoomStoryOrganiser*)[subThread rootProxy];
+	
+	[(NSDistantObject*)rootProxy setProtocolForProxy: @protocol(ZoomStoryIDFetcherProtocol)];
+	[(ZoomStoryOrganiser*)rootProxy startedActing];
 			
 	// Preference keys indicate the filenames
 	NSEnumerator* filenameEnum = [prefs keyEnumerator];
 	NSString* filename;
 	
+	NSAutoreleasePool* p2 = [[NSAutoreleasePool alloc] init];
 	while (filename = [filenameEnum nextObject]) {
+		[p2 release];
+		p2 =  [[NSAutoreleasePool alloc] init];
+		
 		NSData* storyData = [prefs objectForKey: filename];
 		ZoomStoryID* fileID = [NSUnarchiver unarchiveObjectWithData: storyData];
-		ZoomStoryID* realID = [[ZoomStoryID alloc] initWithZCodeFile: filename];
+		ZoomStoryID* realID = [rootProxy idForFile: filename];
+		//ZoomStoryID* realID = [ZoomStoryID idForFile: filename];
+		//ZoomStoryID* realID = [[ZoomStoryID alloc] initWithZCodeFile: filename];
 		
 		if (fileID != nil && realID != nil && [fileID isEqual: realID]) {
 			// Check for a pre-existing entry
@@ -99,13 +172,21 @@ static NSString* ZoomIdentityFilename = @".zoomIdentity";
 			
 			// Remove old entries
 			if (oldFilename) {
+				int index = [storyFilenames indexOfObject: oldFilename];
+				
 				[identsToFilenames removeObjectForKey: fileID];
-				[storyFilenames removeObject: oldFilename];
+				
+				[storyFilenames removeObjectAtIndex: index];
+				[storyIdents removeObjectAtIndex: index];
 			}
 			
 			if (oldIdent) {
+				int index = [storyIdents indexOfObject: oldIdent];
+
 				[filenamesToIdents removeObjectForKey: filename];
-				[storyIdents removeObject: oldIdent];
+				
+				[storyFilenames removeObjectAtIndex: index];
+				[storyIdents removeObjectAtIndex: index];
 			}
 			
 			// Add this entry
@@ -115,22 +196,22 @@ static NSString* ZoomIdentityFilename = @".zoomIdentity";
 			[storyFilenames addObject: newFilename];
 			[storyIdents addObject: newIdent];
 			
-			[identsToFilenames setObject: newFilename forKey: newIdent];
-			[filenamesToIdents setObject: newIdent forKey: newFilename];
-			
+			if (newIdent != nil) {
+				[identsToFilenames setObject: newFilename forKey: newIdent];
+				[filenamesToIdents setObject: newIdent forKey: newFilename];
+			}
+
 			[storyLock unlock];
 		}
 		
-		[realID release];
-		
 		counter++;
 		if (counter > 40) {
 			counter = 0;
-			[(ZoomStoryOrganiser*)[subThread rootProxy] organiserChanged];
+			[rootProxy organiserChanged];
 		}
 	}	
 	
-	[(ZoomStoryOrganiser*)[subThread rootProxy] organiserChanged];
+	[rootProxy organiserChanged];
 	
 	// If story organisation is on, we need to check for any disappeared stories that have appeared in
 	// the organiser directory, and recreate any story data as required.
@@ -173,12 +254,33 @@ static NSString* ZoomIdentityFilename = @".zoomIdentity";
 				// Must be a directory
 				if (gEnt->d_type != DT_DIR) continue;
 				
-				// See if there's a game.z5 there
+				// See if there's a story file there
 				NSString* gameDir = [newDir stringByAppendingPathComponent: gameName];
-				NSString* gameFile = [gameDir stringByAppendingPathComponent: @"game.z5"];
+				NSString* gameFile = nil;
+				ZoomStoryID* gameFileID = nil;
+				
+				// Iterate through the files in this directory
+				DIR* gameD = opendir([gameDir UTF8String]);
+				struct dirent* gameEnt;
+				
+				while (gameD && (gameEnt = readdir(gameD))) {
+					NSString* gameFileName = [NSString stringWithUTF8String: gameEnt->d_name];
+					if ([gameFileName isEqualToString: @".."] ||
+						[gameFileName hasPrefix: @"."]) {
+						continue;
+					}
+					
+					gameFileID = [rootProxy idForFile: [gameDir stringByAppendingPathComponent: gameFileName]];
+					if (gameFileID != nil) {
+						gameFile = [gameDir stringByAppendingPathComponent: gameFileName];
+						break;
+					}
+				}
+				
+				if (gameD) closedir(gameD);
 				
 				struct stat sb;
-				if (stat([gameFile UTF8String], &sb) != 0) continue;
+				if (gameFile == nil || stat([gameFile UTF8String], &sb) != 0) continue;
 				
 				// See if it's already in our database
 				[storyLock lock];
@@ -199,10 +301,10 @@ static NSString* ZoomIdentityFilename = @".zoomIdentity";
 		if (orgD) closedir(orgD);
 	}
 
-	[(ZoomStoryOrganiser*)[subThread rootProxy] organiserChanged];
+	[rootProxy organiserChanged];
 
 	// Tidy up
-	[(ZoomStoryOrganiser*)[subThread rootProxy] endedActing];
+	[rootProxy endedActing];
 
 	[subThread release];
 	[port1 release];
@@ -216,6 +318,7 @@ static NSString* ZoomIdentityFilename = @".zoomIdentity";
 	[self release];
 	
 	// Clear the pool
+	[p2 release];
 	[p release];
 }
 
@@ -254,7 +357,7 @@ static NSString* ZoomIdentityFilename = @".zoomIdentity";
 }
 
 - (void) foundFileNotInDatabase: (NSArray*) info {
-	// Called from the preferenceThread when a story not in the database is found
+	// Called from the preferenceThread (on the main thread) when a story not in the database is found
 	NSString* groupName = [info objectAtIndex: 0];
 	NSString* gameName = [info objectAtIndex: 1];
 	NSString* gameFile = [info objectAtIndex: 2];
@@ -265,7 +368,7 @@ static NSString* ZoomIdentityFilename = @".zoomIdentity";
 	}
 	
 	// Check for story metadata first
-	ZoomStoryID* newID = [[[ZoomStoryID alloc] initWithZCodeFile: gameFile] autorelease];
+	ZoomStoryID* newID = [ZoomStoryID idForFile: gameFile];
 	
 	if (newID == nil) {
 		NSLog(@"Found unindexed game at %@, but failed to obtain an ID. Not indexing", gameFile);
@@ -292,14 +395,9 @@ static NSString* ZoomIdentityFilename = @".zoomIdentity";
 	if (oldStory == nil) {
 		NSLog(@"Creating metadata entry for story '%@'", gameName);
 		
-		ZoomStory* newStory = [[ZoomStory alloc] init];
+		ZoomStory* newStory = [[ZoomStory defaultMetadataForFile: gameFile] retain];
 		
-		[newStory setTitle: gameName];
-		if (![groupName isEqualToString: @"Ungrouped"]);
-		[newStory setGroup: groupName];
-		[newStory addID: newID];
-		
-		[data storeStory: newStory];
+		[data copyStory: newStory];
 		[data writeToDefaultFile];
 		oldStory = [newStory autorelease];
 	} else {
@@ -319,7 +417,7 @@ static NSString* ZoomIdentityFilename = @".zoomIdentity";
 			[oldStory setObject: possibleResource
 						 forKey: @"ResourceFilename"];
 
-			[data storeStory: oldStory];
+			[data copyStory: oldStory];
 			[data writeToDefaultFile];
 		}
 	}
@@ -407,7 +505,8 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 		  organise: NO];
 }
 
-- (void) removeStoryWithIdent: (ZoomStoryID*) ident {
+- (void) removeStoryWithIdent: (ZoomStoryID*) ident
+		   deleteFromMetadata: (BOOL) delete {
 	[storyLock lock];
 	
 	NSString* filename = [identsToFilenames objectForKey: ident];
@@ -415,8 +514,17 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 	if (filename != nil) {
 		[filenamesToIdents removeObjectForKey: filename];
 		[identsToFilenames removeObjectForKey: ident];
-		[storyIdents removeObjectIdenticalTo: ident];
-		[storyFilenames removeObject: filename];
+		
+		int index = [storyFilenames indexOfObject: filename];
+		if (index != NSNotFound) {
+			[storyIdents removeObjectAtIndex: index];
+			[storyFilenames removeObjectAtIndex: index];			
+		}
+	}
+	
+	if (delete) {
+		[[[NSApp delegate] userMetadata] removeStoryWithIdent: ident];
+		[[[NSApp delegate] userMetadata] writeToDefaultFile];
 	}
 	
 	[storyLock unlock];
@@ -425,7 +533,9 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 
 - (void) addStory: (NSString*) filename
 		withIdent: (ZoomStoryID*) ident
-		 organise: (BOOL) organise {	
+		 organise: (BOOL) organise {
+	if (ident == nil) return;
+	
 	[storyLock lock];
 	
 	NSString* oldFilename;
@@ -437,21 +547,37 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 	// Get the story from the metadata database
 	ZoomStory* theStory = [[NSApp delegate] findStory: ident];
 	
+#if DEVELOPMENT_BUILD
+	NSLog(@"Adding %@ (IFID %@)", filename, ident);
+	if (oldFilename) {
+		NSLog(@"... previously %@ (%@)", oldFilename, oldIdent);
+	}
+#endif
+	
 	// If there's no story registered, then we need to create one
 	if (theStory == nil) {
-		theStory = [[ZoomStory alloc] init];
+		// theStory = [[[NSApp delegate] userMetadata] findOrCreateStory: ident];
+		Class pluginClass = [[ZoomPlugInManager sharedPlugInManager] plugInForFile: filename];
+		ZoomPlugIn* pluginInstance = pluginClass?[[pluginClass alloc] initWithFilename: filename]:nil;
 		
-		[theStory addID: ident];
+		if (pluginInstance) {
+			theStory = [[pluginInstance autorelease] defaultMetadata];
+		} else {
+			theStory = [ZoomStory defaultMetadataForFile: filename];
+		}
+		
+		[[[NSApp delegate] userMetadata] copyStory: theStory];
 		[theStory setTitle: [[filename lastPathComponent] stringByDeletingPathExtension]];
 		
-		[[[NSApp delegate] userMetadata] storeStory: [theStory autorelease]];
 		[[[NSApp delegate] userMetadata] writeToDefaultFile];
 	}
 		
 	if (oldFilename && oldIdent && [oldFilename isEqualToString: filename] && [oldIdent isEqualTo: ident]) {
 		// Nothing to do
 		[storyLock unlock];
-
+#if DEVELOPMENT_BUILD
+		NSLog(@"... looks OK");
+#endif
 		
 		if (organise) {
 			[self organiseStory: theStory
@@ -460,16 +586,31 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 		return;
 	}
 	
+	[[ident retain] autorelease];
+	[[filename retain] autorelease];
+	[[oldFilename retain] autorelease];
+	[[oldIdent retain] autorelease];
+	
 	if (oldFilename) {
 		[identsToFilenames removeObjectForKey: ident];
 		[filenamesToIdents removeObjectForKey: oldFilename];
-		[storyFilenames removeObject: oldFilename];
+		
+		int index = [storyFilenames indexOfObject: oldFilename];
+		if (index != NSNotFound) {
+			[storyFilenames removeObjectAtIndex: index];			
+			[storyIdents removeObjectAtIndex: index];
+		}
 	}
 
 	if (oldIdent) {
 		[filenamesToIdents removeObjectForKey: filename];
 		[identsToFilenames removeObjectForKey: oldIdent];
-		[storyIdents removeObject: oldIdent];
+	
+		int index = [storyIdents indexOfObject: oldIdent];
+		if (index != NSNotFound) {
+			[storyFilenames removeObjectAtIndex: index];			
+			[storyIdents removeObjectAtIndex: index];
+		}
 	}
 	
 	[filenamesToIdents removeObjectForKey: filename];
@@ -481,8 +622,12 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 	[storyFilenames addObject: newFilename];
 	[storyIdents addObject: newIdent];
 	
-	[identsToFilenames setObject: newFilename forKey: newIdent];
-	[filenamesToIdents setObject: newIdent forKey: newFilename];
+	NSLog(@"... now %@ (%@)", newFilename, newIdent);
+	
+	if (newIdent != nil) {
+		[identsToFilenames setObject: newFilename forKey: newIdent];
+		[filenamesToIdents setObject: newIdent forKey: newFilename];
+	}
 	
 	[storyLock unlock];
 	
@@ -752,7 +897,7 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 	// If there is a directory in the preferences, then that's the directory to use
 	NSDictionary* gameDirs = [defaults objectForKey: ZoomGameDirectories];
 	
-	if (gameDirs)
+	if (gameDirs && [ident description] != nil)
 		confDir = [gameDirs objectForKey: [ident description]];
 
 	BOOL isDir;
@@ -782,10 +927,12 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 		newGameDirs = [[NSMutableDictionary alloc] init];
 	}
 
-	[newGameDirs setObject: gameDir
-					forKey: [ident description]];
-	[defaults setObject: [newGameDirs autorelease]
-				 forKey: ZoomGameDirectories];
+	if (ident != nil && [ident description] != nil) {
+		[newGameDirs setObject: gameDir
+						forKey: [ident description]];
+		[defaults setObject: [newGameDirs autorelease]
+					 forKey: ZoomGameDirectories];
+	}
 	
 	return gameDir;
 }
@@ -797,6 +944,10 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 	currentDir = [currentDir stringByStandardizingPath];
 	
 	if (currentDir == nil) return NO;
+
+#ifdef DEVELOPMENT_BUILD
+	NSLog(@"Moving %@ to its preferred path (currently at %@)", ident, currentDir);
+#endif
 	
 	// Get the 'ideal' directory
 	NSString* idealDir = [self findDirectoryForIdent: ident
@@ -808,6 +959,10 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 	if ([[idealDir lowercaseString] isEqualToString: [currentDir lowercaseString]]) 
 		return YES;
 	
+#ifdef DEVELOPMENT_BUILD
+	NSLog(@"Ideal location is %@", idealDir);
+#endif
+	
 	// If they don't match, then idealDir should be new (or something weird has just occured)
 	// Hmph. HFS+ is case-insensitve, and stringByStandardizingPath does not take account of this. This could
 	// cause some major problems with organiseStory:withIdent:, as that deletes/copies files...
@@ -843,10 +998,12 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 		newGameDirs = [[NSMutableDictionary alloc] init];
 	}
 	
-	[newGameDirs setObject: idealDir
-					forKey: [ident description]];
-	[defaults setObject: [newGameDirs autorelease]
-				 forKey: ZoomGameDirectories];	
+	if (ident != nil && [ident description] != nil) {
+		[newGameDirs setObject: idealDir
+						forKey: [ident description]];
+		[defaults setObject: [newGameDirs autorelease]
+					 forKey: ZoomGameDirectories];	
+	}
 	
 	return YES;
 }
@@ -854,6 +1011,10 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 - (void) someStoryHasChanged: (NSNotification*) not {
 	ZoomStory* story = [not object];
 	
+#ifdef DEVELOPMENT_BUILD
+	NSLog(@"Story %@ has changed", [story title]);
+#endif
+	
 	if (![story isKindOfClass: [ZoomStory class]]) {
 		NSLog(@"someStoryHasChanged: called with a non-story object (too many spoons?)");
 		return; // Unlikely but possible. If I'm a spoon, that is.
@@ -873,10 +1034,15 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 
 - (void) finishChangingStory: (ZoomStory*) story {
 	// For our pre-arranged stories, several IDs are possible, but more usually one
-	NSEnumerator* identEnum = [[story storyIDs] objectEnumerator];
+	NSArray* storyIDs = [story storyIDs];
+	NSEnumerator* identEnum = [storyIDs objectEnumerator];
 	ZoomStoryID* ident;
 	BOOL changed = NO;
 	
+#ifdef DEVELOPMENT_BUILD
+	NSLog(@"Finishing changing %@", [story title]);
+#endif
+	
 	while (ident = [identEnum nextObject]) {
 		int identID = [storyIdents indexOfObject: ident];
 		
@@ -884,29 +1050,40 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 			// Get the old location of the game
 			ZoomStoryID* realID = [storyIdents objectAtIndex: identID];
 			
-			NSString* oldGameFile = [self directoryForIdent: ident create: NO];
-			oldGameFile = [oldGameFile stringByAppendingPathComponent: @"game.z5"];
+			NSString* oldGameFile = [self directoryForIdent: ident
+													 create: NO];
 			NSString* oldGameLoc = [storyFilenames objectAtIndex: identID];
+			oldGameFile = [oldGameFile stringByAppendingPathComponent: [oldGameLoc lastPathComponent]];
 			
 			oldGameFile = [oldGameFile stringByStandardizingPath];
 			oldGameLoc = [oldGameLoc stringByStandardizingPath];
-
+			
+#ifdef DEVELOPMENT_BUILD
+			NSLog(@"ID %@ (%@) is located at %@ (%@)", ident, realID, oldGameFile, oldGameLoc);
+#endif
+			
 			// Actually perform the move
 			if ([self moveStoryToPreferredDirectoryWithIdent: [storyIdents objectAtIndex: identID]]) {
 				changed = YES;
 			
 				// Store the new location of the game, if necessary
-				if ([oldGameLoc isEqualToString: oldGameFile]) {
-					NSString* newGameFile = [[self directoryForIdent: ident create: NO] stringByAppendingPathComponent: @"game.z5"];
+				if (YES || [oldGameLoc isEqualToString: oldGameFile]) {
+					NSString* newGameFile = [[self directoryForIdent: ident create: NO] stringByAppendingPathComponent: [oldGameLoc lastPathComponent]];
 					newGameFile = [newGameFile stringByStandardizingPath];
-
+					
+#ifdef DEVELOPMENT_BUILD
+					NSLog(@"Have moved to %@", newGameFile);
+#endif
+					
 					if (![oldGameFile isEqualToString: newGameFile]) {
 						[filenamesToIdents removeObjectForKey: oldGameFile];
 						
-						[filenamesToIdents setObject: realID
-											  forKey: newGameFile];
-						[identsToFilenames setObject: newGameFile
-											  forKey: realID];
+						if (realID != nil) {
+							[filenamesToIdents setObject: realID
+												  forKey: newGameFile];
+							[identsToFilenames setObject: newGameFile
+												  forKey: realID];
+						}
 						
 						[storyFilenames replaceObjectAtIndex: identID
 												  withObject: newGameFile];
@@ -930,16 +1107,30 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 		NSLog(@"WARNING: Attempted to organise a story with no filename");
 		return;
 	}
-		
+	
+#if DEVELOPMENT_BUILD
+	NSLog(@"Organising %@ (%@)", [story title], ident);
+#endif
+	
+	[storyLock lock];
+	
 	NSString* oldFilename = [[filename retain] autorelease];
+
+#if DEVELOPMENT_BUILD
+	NSLog(@"... currently at %@", oldFilename);
+#endif
 	
 	// Copy to a standard directory, change the filename we're using
 	filename = [filename stringByStandardizingPath];
 		
 	NSString* fileDir = [self directoryForIdent: ident create: YES];
-	NSString* destFile = [fileDir stringByAppendingPathComponent: @"game.z5"];
+	NSString* destFile = [fileDir stringByAppendingPathComponent: [oldFilename lastPathComponent]];
 	destFile = [destFile stringByStandardizingPath];
-		
+	
+#if DEVELOPMENT_BUILD
+	NSLog(@"... best directory %@ (file will be %@)", fileDir, destFile);
+#endif
+	
 	if (![filename isEqualToString: destFile]) {
 		BOOL moved = NO;
 		
@@ -985,13 +1176,17 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 		}
 		
 		if (!outsideOrganisation) {
-			// Have to move the file from the directory it's in to the new directory
+			// Have to move the file from the directory its in to the new directory
 			// Really want to move resources and savegames too... Hmm
 			NSString* oldDir = [filename stringByDeletingLastPathComponent];
 			NSEnumerator* dirEnum = [[[NSFileManager defaultManager] directoryContentsAtPath: oldDir] objectEnumerator];
 			
 			NSString* fileToMove;
 			while (fileToMove = [dirEnum nextObject]) {
+#if DEVELOPMENT_BUILD
+				NSLog(@"... reorganising %@ to %@", [oldDir stringByAppendingPathComponent: fileToMove], [fileDir stringByAppendingPathComponent: fileToMove]);
+#endif
+
 				[[NSFileManager defaultManager] movePath: [oldDir stringByAppendingPathComponent: fileToMove]
 												  toPath: [fileDir stringByAppendingPathComponent: fileToMove]
 												 handler: nil];
@@ -1019,14 +1214,26 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 	}
 	
 	// Update the indexes
-	[identsToFilenames setObject: filename
-						  forKey: ident];
-	[filenamesToIdents removeObjectForKey: oldFilename];
-	[filenamesToIdents setObject: ident
-						  forKey: filename];
-	
-	[storyFilenames removeObject: oldFilename];
-	[storyFilenames addObject: filename];
+	int filenameIndex = [storyFilenames indexOfObject: oldFilename];
+	if (filenameIndex != NSNotFound) {
+		[storyFilenames removeObjectAtIndex: filenameIndex];
+		[storyIdents removeObjectAtIndex: filenameIndex];
+	}
+
+	if (ident != nil) {
+#if DEVELOPMENT_BUILD
+		NSLog(@"... %@ <=> %@", ident, filename);
+#endif		
+		
+		[identsToFilenames setObject: filename
+							  forKey: ident];
+		[filenamesToIdents removeObjectForKey: oldFilename];
+		[filenamesToIdents setObject: ident
+							  forKey: filename];
+		
+		[storyFilenames addObject: filename];
+		[storyIdents	addObject: ident];
+	}
 	
 	// Organise the story's resources
 	NSString* resources = [story objectForKey: @"ResourceFilename"];
@@ -1038,6 +1245,7 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 		
 		if (dir == nil) {
 			NSLog(@"No organised directory for game: cannot store resources");
+			[storyLock unlock];
 			return;
 		}
 		
@@ -1049,21 +1257,34 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 		}
 		
 		NSString* newFile = [dir stringByAppendingPathComponent: @"resource.blb"];
+		NSString* oldFile = resources;
 		
-		if (![fm copyPath: resources
-				   toPath: newFile
-				  handler: nil]) {
-			NSLog(@"Unable to copy resource file to new location");
-		} else {
-			resources = newFile;
+		newFile = [newFile stringByStandardizingPath];
+		oldFile = [oldFile stringByStandardizingPath];
+		
+		if (![[oldFile lowercaseString] isEqualToString: [newFile lowercaseString]]) {
+			if ([fm fileExistsAtPath: newFile]) {
+				[fm removeFileAtPath: newFile
+							 handler: nil];
+			}
+			
+			if (![fm copyPath: resources
+					   toPath: newFile
+					  handler: nil]) {
+				NSLog(@"Unable to copy resource file to new location");
+			} else {
+				resources = newFile;
+			}
+			
+			[story setObject: resources
+					  forKey: @"ResourceFilename"];
 		}
-		
-		[story setObject: resources
-				  forKey: @"ResourceFilename"];
 	} else {
 		[story setObject: nil
 				  forKey: @"ResourceFilename"];
 	}
+
+	[storyLock unlock];
 }
 
 - (void) organiseStory: (ZoomStory*) story {
@@ -1127,7 +1348,9 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 
 - (void) renamedIdent: (ZoomStoryID*) ident
 		   toFilename: (NSString*) filename {
-	filename = [[filename copy] autorelease];
+	if (ident == nil) return;
+	
+	filename = [NSString stringWithString: filename];
 	
 	[storyLock lock];
 	
@@ -1331,9 +1554,9 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
                                 forMode: NSDefaultRunLoopMode];
 	NSConnection* subThreadConnection = [[NSConnection allocWithZone: [self zone]] initWithReceivePort: threadPort2
 																							  sendPort: threadPort1];
-	[subThreadConnection setRootObject: self];
 	
 	// Start things rolling
+	[[subThreadConnection rootProxy] setProtocolForProxy: @protocol(ZoomStoryIDFetcherProtocol)];
 	[(ZoomStoryOrganiser*)[subThreadConnection rootProxy] startedActing];
 	
 	NSString* gameStorageDirectory = [[[(ZoomStoryOrganiser*)[subThreadConnection rootProxy] gameStorageDirectory] copy] autorelease];
@@ -1566,8 +1789,17 @@ static ZoomStoryOrganiser* sharedOrganiser = nil;
 			}
 			
 			// Change the storyFilenames array
-			[storyFilenames removeObject: filename];
+			/* -- ??
+			int oldIndex = [storyFilenames indexOfObject: filename];
+			
+			if (oldIndex != NSNotFound) {
+				[storyFilenames removeObjectAtIndex: oldIndex];
+				[storyIdents removeObjectAtIndex: oldIndex];
+			}
+			 */
+			
 			[storyFilenames addObject: [titleDirectory stringByAppendingPathComponent: [filename lastPathComponent]]];
+			//[storyIdents addObject: storyID];
 
 			[storyLock unlock];
 			
