@@ -19,7 +19,10 @@
 
 // Sets variables to force extreme memory checking in the Zoom task; this provides a fairly huge performance
 // decrease, but provides 'earliest possible' warning of heap corruption.
-#undef ZoomTaskMaximumMemoryDebug
+#undef  ZoomTaskMaximumMemoryDebug
+
+// Turn on tracing of text editing events
+#undef  ZoomTraceTextEditing
 
 @implementation ZoomView
 
@@ -34,7 +37,7 @@ static void finalizeViews(void);
     atexit(finalizeViews);
 }
 
-+ (void) finalize {
++ (void) selfDestruct {
     int view;
     
     for (view=0;view<nAllocatedViews;view++) {
@@ -43,7 +46,7 @@ static void finalizeViews(void);
 }
 
 static void finalizeViews(void) {
-    [ZoomView finalize];
+    [ZoomView selfDestruct];
 }
 
 - (id)initWithFrame:(NSRect)frame {
@@ -240,6 +243,8 @@ static void finalizeViews(void) {
 	if (resources) [resources release];
 	
 	if (terminatingChars) [terminatingChars release];
+	
+	if (originalFonts) [originalFonts release];
 
     [super dealloc];
 }
@@ -251,6 +256,9 @@ static void finalizeViews(void) {
 		NSImage* pixmap = [pixmapWindow pixmap];
 		NSSize pixSize = [pixmap size];
 		
+		NSRect realFrame = [self convertRect: bounds 
+									  toView: [[self window] contentView]];
+		
 		/*
 		[pixmap drawAtPoint: NSMakePoint(floor(bounds.origin.x + (bounds.size.width-pixSize.width)/2.0), floor(bounds.origin.y + (bounds.size.height-pixSize.height)/2.0))
 				   fromRect: NSMakeRect(0,0,pixSize.width, pixSize.height)
@@ -267,6 +275,8 @@ static void finalizeViews(void) {
 		
 		[[NSGraphicsContext currentContext] saveGraphicsState];
 		[invertTransform invert];
+		[invertTransform translateXBy: realFrame.origin.x
+								  yBy: realFrame.origin.y];
 		[invertTransform set];
 
 		[pixmap drawInRect: bounds
@@ -291,15 +301,45 @@ static void finalizeViews(void) {
 // Scaling
 - (void) setScaleFactor: (float) scaling {
 	scaleFactor = scaling;
-	[textScroller setScaleFactor: scaling];
-		
-	NSRect tVF = [textView frame];
-	NSRect tVB = tVF;
-	tVB.origin.x = tVB.origin.y = 0;
-	tVB.size.width *= scaling;
-	tVB.size.height *= scaling;
+	[textView setPastedLineScaleFactor: 1.0/scaling];
 	
-	[textView setBounds: tVB];
+	// Scale up all the fonts
+	// Previously we used OS X scaling, but this is problematic (largely because NSLayoutManager is fragile)
+	
+	if (scaling == 1.0) {
+		// Scale factor of 1 is a special case: restore the standard fonts
+		if (originalFonts) {
+			[fonts release];
+			fonts = originalFonts;
+			originalFonts = nil;
+		}
+	} else {
+		// If we don't currently know the 'original' fonts, set them to the current fonts
+		// This will cause errors if somehow the scale factor gets set without the original
+		// fonts being copied
+		if (!originalFonts) {
+			originalFonts = [fonts copy];
+		}
+		
+		// Scale up all of the fonts
+		NSMutableArray* newFonts = [[NSMutableArray alloc] init];
+		NSEnumerator* fontEnum = [originalFonts objectEnumerator];
+		NSFont* origFont;
+		
+		while (origFont = [fontEnum nextObject]) {
+			NSFont* scaledFont = [[NSFontManager sharedFontManager] convertFont: origFont
+																		 toSize: [origFont pointSize] / scaleFactor];
+			
+			[newFonts addObject: scaledFont];
+		}
+		
+		// Done
+		[fonts release];
+		fonts = newFonts;
+	}
+	
+	[self reformatWindow];
+	[self scrollToEnd];
 	
 	if (zMachine) {
 		[zMachine displaySizeHasChanged];
@@ -322,6 +362,14 @@ static void finalizeViews(void) {
 
 // = ZDisplay functions =
 
+- (int) interpreterVersion {
+	return [viewPrefs interpreter];
+}
+
+- (int) interpreterRevision {
+	return [viewPrefs revision];
+}
+
 - (void) beep {
 	// All the sound we support at the moment
 	if (delegate && [delegate respondsToSelector: @selector(beep)]) {
@@ -404,7 +452,30 @@ static void finalizeViews(void) {
     exclusiveMode = NO;
 }
 
+- (float) bufferHeight {
+	NSLayoutManager* mgr = [textView layoutManager];
+	NSTextStorage* textStorage = [textView textStorage];
+	
+	NSRange lastGlyph = [mgr glyphRangeForCharacterRange: NSMakeRange(0, [textStorage length])
+									actualCharacterRange: nil];
+	
+	if (lastGlyph.location + lastGlyph.length > 0) {
+		return NSMaxY([mgr boundingRectForGlyphRange: lastGlyph
+									 inTextContainer: [textView textContainer]]);
+	}
+	
+	return 0;
+}
+
 - (void) flushBuffer: (ZBuffer*) toFlush {
+#ifdef ZoomTraceTextEditing
+	NSLog(@"Begin editing: flushBuffer");
+#endif
+	
+	[[textView layoutManager] setBackgroundLayoutEnabled: NO];
+	BOOL truncated = NO;
+	float oldHeight = [self bufferHeight];
+
 	[[textView textStorage] beginEditing];
 	editingTextView = YES;
 
@@ -421,11 +492,23 @@ static void finalizeViews(void) {
 			// Need to truncate
 			[[textView textStorage] deleteCharactersInRange: NSMakeRange(0, len - preserve)];
 			inputPos -= len-preserve;
+			truncated = YES;
 		}
 	}
 	
+#ifdef ZoomTraceTextEditing
+	NSLog(@"End editing: flushBuffer");
+#endif
 	[[textView textStorage] endEditing];
 	editingTextView = NO;
+	
+	float newHeight = [self bufferHeight];
+	if (newHeight != oldHeight && truncated) {
+		[textView offsetPastedLines: oldHeight-newHeight];
+	}
+	
+	if (willScrollToEnd) [self scrollToEnd];
+	[[textView layoutManager] setBackgroundLayoutEnabled: YES];
 }
 
 // Set whether or not we recieve certain types of data
@@ -495,6 +578,10 @@ static void finalizeViews(void) {
 		
 		if (nextInput == nil) {
 			// End of input
+			if (delegate && [delegate respondsToSelector: @selector(inputSourceHasFinished:)]) {
+				[delegate inputSourceHasFinished: inputSource];
+			}
+			
 			[inputSource release];
 			inputSource = nil;
 		} else {			
@@ -609,6 +696,10 @@ static void finalizeViews(void) {
 		
 		if (nextInput == nil) {
 			// End of input
+			if (delegate && [delegate respondsToSelector: @selector(inputSourceHasFinished:)]) {
+				[delegate inputSourceHasFinished: inputSource];
+			}
+			
 			[inputSource release];
 			inputSource = nil;
 		} else {
@@ -709,38 +800,60 @@ static void finalizeViews(void) {
 }
 
 - (void) scrollToEnd {
+	BOOL wasEditingTextView = editingTextView;
+
 	if (editingTextView) {
 		// Re-queue for later
 		if (!willScrollToEnd) {
-			[[NSRunLoop currentRunLoop] performSelector: @selector(scrollToEnd)
-												 target: self
-											   argument: nil
-												  order: 64
-												  modes: [NSArray arrayWithObject: NSDefaultRunLoopMode]];
 			willScrollToEnd = YES;
 		}
 
 		return;
 	}
 	
+	if (wasEditingTextView) {
+#ifdef ZoomTraceTextEditing
+		NSLog(@"End editing: scroll to end");
+#endif
+		editingTextView = NO;
+		[[textView textStorage] endEditing];
+	}
+	
 	willScrollToEnd = NO;
 	
-    NSLayoutManager* mgr = [textView layoutManager];
+	if ([[textView textStorage] length] <= 0) {
+		// No scrolling to do if the view is empty
+		if (wasEditingTextView && !editingTextView) {
+#ifdef ZoomTraceTextEditing
+			NSLog(@"Begin editing: scroll to end");
+#endif
+			editingTextView = YES;
+			[[textView textStorage] beginEditing];
+		}
+		return;
+	}
 	
-    NSRange endGlyph = [textView selectionRangeForProposedRange:
-        NSMakeRange([[textView textStorage] length]-1, 1)
-                                                           granularity: NSSelectByCharacter];
-    if (endGlyph.location > 0xf0000000) {
-        return; // Doesn't exist
-    }
-    
-    NSRect endRect = [mgr boundingRectForGlyphRange: endGlyph
-                                    inTextContainer: [textView textContainer]];
-	NSRect frame = [textView convertRect: [[textScroller contentView] frame]
-								fromView: textScroller];
+    NSLayoutManager* mgr = [textView layoutManager];
+	NSTextStorage* textStorage = [textView textStorage];
+	
+	NSRange lastGlyph = [mgr glyphRangeForCharacterRange: NSMakeRange(0, [textStorage length])
+									actualCharacterRange: nil];
+	
+	// Force the layout manager to lay out the final glyph so we can scroll there
+	if (lastGlyph.location + lastGlyph.length > 0) {
+		[mgr boundingRectForGlyphRange: lastGlyph
+					   inTextContainer: [textView textContainer]];
+	}
+	
+	[textView scrollPoint: NSMakePoint(0, NSMaxY([textView bounds]))];
 
-    [textView scrollPoint: NSMakePoint(0,
-									   NSMaxY(endRect) - frame.size.height)];
+	if (wasEditingTextView && !editingTextView) {
+#ifdef ZoomTraceTextEditing
+		NSLog(@"Begin editing: scroll to end");
+#endif
+		editingTextView = YES;
+		[[textView textStorage] beginEditing];
+	}
 }
 
 - (void) displayMore: (BOOL) shown {
@@ -773,6 +886,11 @@ static void finalizeViews(void) {
 		}
 	}
 	
+	if ([[textView textStorage] length] <= 0) {
+		// Nothing to do
+		return;
+	}
+	
 	// Find the last glyph in the text view
     NSRange endGlyph = [textView selectionRangeForProposedRange:
         NSMakeRange([[textView textStorage] length]-1, 1)
@@ -814,7 +932,7 @@ static void finalizeViews(void) {
 - (void) resetMorePrompt {
     // Resets the point at which paging will next occur
     // Does NOT reset the point if paging is already going on
-    
+	
     double maxHeight;
     NSLayoutManager* mgr = [textView layoutManager];
 
@@ -822,9 +940,26 @@ static void finalizeViews(void) {
         return; // More prompt is currently being displayed
     }
 
-    NSRange endGlyph = [textView selectionRangeForProposedRange:
-        NSMakeRange([[textView textStorage] length]-1, 1)
-                                                    granularity: NSSelectByCharacter];
+	BOOL wasEditingTextView = editingTextView;
+	editingTextView = NO;
+    if (wasEditingTextView) {
+#ifdef ZoomTraceTextEditing
+		NSLog(@"End editing: reset more prompt");
+#endif
+		[[textView textStorage] endEditing];
+		if (willScrollToEnd) [self scrollToEnd];
+	}
+
+    NSRange endGlyph;
+	int length = [[textView textStorage] length];
+	
+	if (length > 0) {
+		endGlyph = [textView selectionRangeForProposedRange:
+			NSMakeRange([[textView textStorage] length]-1, 1)
+												   granularity: NSSelectByCharacter];
+	} else {
+		endGlyph = NSMakeRange(0xffffffff,0);
+	}
     if (endGlyph.location < 0xf0000000) {
         NSRect endRect = [mgr boundingRectForGlyphRange: endGlyph
                                         inTextContainer: [textView textContainer]];
@@ -837,12 +972,27 @@ static void finalizeViews(void) {
     maxHeight += [textScroller contentSize].height;
 
     [textView setMaxSize: NSMakeSize(1e8, maxHeight)];
+	
     [self scrollToEnd];
+	
+    if (wasEditingTextView) {
+#ifdef ZoomTraceTextEditing
+		NSLog(@"Begin editing: reset more prompt");
+#endif
+		[[textView textStorage] beginEditing];
+		editingTextView = YES;
+	}
 }
 
 - (void) updateMorePrompt {
 	if (pixmapWindow) return; // Nothing to do
 	
+	BOOL wasEditingTextView = editingTextView;
+	if (wasEditingTextView) {
+		editingTextView = NO;
+		[[textView textStorage] endEditing]; 
+	}
+	
     // Updates the more prompt to represent the new height of the window
 	NSSize contentSize = [textScroller contentSize];
 	contentSize = [textView convertSize: contentSize
@@ -854,6 +1004,11 @@ static void finalizeViews(void) {
     [textView sizeToFit];
     [self scrollToEnd];
     [self displayMoreIfNecessary];
+
+	if (wasEditingTextView) {
+		editingTextView = YES;
+		[[textView textStorage] beginEditing]; 
+	}
 }
 
 - (void) page {
@@ -1200,6 +1355,7 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
         fontToUse, NSFontAttributeName,
         foregroundColour, NSForegroundColorAttributeName,
         backgroundColour, NSBackgroundColorAttributeName,
+		[NSNumber numberWithBool: [viewPrefs useLigatures]], NSLigatureAttributeName,
 		[[style copy] autorelease], ZoomStyleAttributeName,
         nil];
 	
@@ -1269,11 +1425,11 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
     // FIXME: check that fonts is valid
 	// FIXME: better to do this with preferences now, but Inform still uses these calls
     
-    [fonts release];
-    fonts = [[NSArray allocWithZone: [self zone]] initWithArray: newFonts 
-                                                      copyItems: YES];
-	
-	[self reformatWindow];
+    [originalFonts release];
+    originalFonts = [[NSArray allocWithZone: [self zone]] initWithArray: newFonts 
+															  copyItems: YES];
+
+	[self setScaleFactor: scaleFactor];
 }
 
 - (void) setColours: (NSArray*) newColours {
@@ -1357,10 +1513,25 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 
 - (void) setUpperBuffer: (double) bufHeight {
     // Update the upper window buffer
-	if (editingTextView) [[textView textStorage] endEditing];
+	BOOL wasEditingTextView = editingTextView;
+	editingTextView = NO;
+	if (wasEditingTextView) {
+#ifdef ZoomTraceTextEditing
+		NSLog(@"End editing: set upper buffer");
+#endif
+		[[textView textStorage] endEditing];
+		if (willScrollToEnd) [self scrollToEnd];
+	}
     NSSize contentSize = [textScroller contentSize];
     [upperWindowBuffer setContainerSize: NSMakeSize(contentSize.width*scaleFactor, bufHeight)];
-	if (editingTextView) [[textView textStorage] beginEditing];
+	if (wasEditingTextView) {
+		if (willScrollToEnd) [self scrollToEnd];
+#ifdef ZoomTraceTextEditing
+		NSLog(@"Begin editing: set upper buffer");
+#endif
+		[[textView textStorage] beginEditing];
+		editingTextView = YES;
+	}
 }
 
 - (double) upperBufferHeight {
@@ -1371,7 +1542,15 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
     int newSize = [self upperWindowSize];
     if (newSize != lastUpperWindowSize) {
 		// Stop editing the text view (if we are editing it)
-		if (editingTextView) [[textView textStorage] endEditing];
+		BOOL wasEditingTextView = editingTextView;
+		editingTextView = NO;
+		if (wasEditingTextView) {
+#ifdef ZoomTraceTextEditing
+			NSLog(@"End editing: rearrange upper windows");
+#endif
+			[[textView textStorage] endEditing];
+			if (willScrollToEnd) [self scrollToEnd];
+		}
 		
         // Lay things out
         lastUpperWindowSize = newSize;
@@ -1388,6 +1567,9 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
         double sepHeight = fixedSize.height * (double)newSize;
         sepHeight -= [upperWindowBuffer containerSize].height;
 		
+		if (inputPos > [[textView textStorage] length])
+			inputPos = [[textView textStorage] length];
+		
         if ([[textView textStorage] length] == 0) {
             [[[textView textStorage] mutableString] insertString: @"\n"
 														 atIndex: inputPos];
@@ -1418,7 +1600,13 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
         [self updateMorePrompt];
 
 		// Restart editing the text view (if we are editing it)
-		if (editingTextView) [[textView textStorage] beginEditing];
+		if (wasEditingTextView) {
+#ifdef ZoomTraceTextEditing
+			NSLog(@"Begin editing: rearrange upper windows");
+#endif
+			[[textView textStorage] beginEditing];
+			editingTextView = YES;
+		}
 	}
 
     // Redraw the upper windows if necessary
@@ -1457,8 +1645,16 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
     //
     // Er, right, the code:
     NSTextContainer* theContainer;
+	BOOL wasEditingTextView = editingTextView;
+	editingTextView = NO;
 	
-	if (editingTextView) [[textView textStorage] endEditing];
+	if (wasEditingTextView) {
+#ifdef ZoomTraceTextEditing
+		NSLog(@"End editing: pad to lower window");
+#endif
+		[[textView textStorage] endEditing];
+		if (willScrollToEnd) [self scrollToEnd];
+	}
 
     if (upperWindowBuffer == nil) return;
 
@@ -1473,7 +1669,12 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
             NSMakeRange([[textView textStorage] length]-1, 1)
                                                     granularity: NSSelectByCharacter];
         if (endGlyph.location > 0xf0000000) {
-			if (editingTextView) [[textView textStorage] beginEditing];
+			if (editingTextView) {
+#ifdef ZoomTraceTextEditing
+				NSLog(@"Begin editing: pad to lower window");
+#endif
+				[[textView textStorage] beginEditing];
+			}
             return; // Doesn't exist
         }
 
@@ -1489,7 +1690,13 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
         // I suppose there's an outside chance of an infinite loop here
     } while (theContainer == upperWindowBuffer);
 
-	if (editingTextView) [[textView textStorage] beginEditing];
+	if (wasEditingTextView) {
+#ifdef ZoomTraceTextEditing
+		NSLog(@"Begin editing: pad to lower window");
+#endif
+		[[textView textStorage] beginEditing];
+		editingTextView = YES;
+	}
 }
 
 - (void) runNewServer: (NSString*) serverName {
@@ -1610,6 +1817,10 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 //	     (phut)
 - (void) zoomTaskFinished: (NSNotification*) not {
 	if ([not object] != zoomTask) return; // Not our task
+	
+	// In case we're self-destructing, retain + autorelease
+	[self retain];
+	[self autorelease];
 
 	[[ZoomConnector sharedConnector] removeView: self];
 
@@ -1805,7 +2016,7 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
             typeCode = 'TEXT';
             if (supportsMessage) {
                 [panel setMessage: [NSString stringWithFormat: @"%@ command recording file", saveOpen]];
-                [panel setAllowedFileTypes: [NSArray arrayWithObjects: @"txt", nil]];
+                [panel setAllowedFileTypes: [NSArray arrayWithObjects: @"txt", @"rec", nil]];
             }
             break;
             
@@ -1996,6 +2207,13 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 										forWriting: NO];
 			
 			if (f) {
+				NSData* skeinData = [f dataForFile: @"Skein.skein"];
+				if (skeinData) {
+					if (delegate && [delegate respondsToSelector: @selector(loadedSkeinData:)]) {
+						[delegate loadedSkeinData: skeinData];
+					}
+				}
+				
 				[zMachine promptedFileIs: [f autorelease]
 									size: [f fileSize]];
 			} else {
@@ -2106,16 +2324,29 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 	colours = [[viewPrefs colours] retain];
 	
 	// Switch on the text-to-speech if required
-	if ([viewPrefs speakGameText] && !textToSpeechReceiver) {
+	if (!textToSpeechReceiver) {
 		textToSpeechReceiver = [[ZoomTextToSpeech alloc] init];
 		[self addOutputReceiver: textToSpeechReceiver];
 	}
+	[textToSpeechReceiver setImmediate: [viewPrefs speakGameText]];
 	
-	if (![viewPrefs speakGameText] && textToSpeechReceiver) {
-		[self removeOutputReceiver: textToSpeechReceiver];
-		[textToSpeechReceiver release];
-		textToSpeechReceiver = nil;
+	[textView setTextContainerInset: NSMakeSize([viewPrefs textMargin], [viewPrefs textMargin])]; 
+	[[textView layoutManager] setHyphenationFactor: [viewPrefs useHyphenation]?1:0];
+	[[textView layoutManager] setUsesScreenFonts: [viewPrefs useScreenFonts]];
+	
+	if ([viewPrefs useKerning]) {
+		[textView useStandardKerning: self];
+	} else {
+		[textView turnOffKerning: self];
 	}
+	if ([viewPrefs useLigatures]) {
+		[textView useStandardLigatures: self];
+	} else {
+		[textView turnOffLigatures: self];
+	}
+	
+	[textScroller setUseUpperDivider: [viewPrefs showBorders]];
+	[textScroller tile];
 	
 	[self reformatWindow];
 }
@@ -2129,6 +2360,9 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 	
 	attributedRange.location = 0;
 	
+#ifdef ZoomTraceTextEditing
+	NSLog(@"Begin editing: reformat window");
+#endif
 	[storage beginEditing];
 	
 	while (attributedRange.location < len) {
@@ -2158,6 +2392,9 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 		attributedRange.location += attributedRange.length;
 	}
 	
+#ifdef ZoomTraceTextEditing
+	NSLog(@"End editing: reformat window");
+#endif
 	[storage endEditing];
 	
 	// Reset the background colour of the lower window
@@ -2187,6 +2424,14 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
     }
 }
 
+- (int) foregroundColour {
+	return [viewPrefs foregroundColour];
+}
+
+- (int) backgroundColour {
+	return [viewPrefs backgroundColour];
+}
+
 // = Autosave =
 
 - (BOOL) createAutosaveDataWithCoder: (NSCoder*) encoder {
@@ -2491,7 +2736,7 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 		return YES;
 	}
 	
-	return NO;
+	return [super acceptsFirstResponder];
 }
 
 - (BOOL) becomeFirstResponder {
@@ -2500,7 +2745,7 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 		return YES;
 	}
 	
-	return NO;
+	return [super becomeFirstResponder];
 }
 
 - (BOOL) resignFirstResponder {
@@ -2508,7 +2753,7 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 		[pixmapCursor setFirst: NO];
 	}
 	
-	return YES;
+	return [super resignFirstResponder];
 }
 
 // = Manual input =
@@ -2555,19 +2800,26 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 }
 
 - (NSString*) receivedTextToDate {
+	// Used to retrieve the text received in the view so far: for example, to find out what the user typed prior to a timeout
+	
 	if (inputLine != nil) {
 		return [inputLine inputLine];
 	} else {
 		NSString* str = [[textView textStorage] string];
 		
-		return [str substringWithRange: NSMakeRange(inputPos,
-													[str length]-inputPos)];
+		NSString* res = [str substringWithRange: NSMakeRange(inputPos,
+															 [str length]-inputPos)];
+		
+		// FIXME: might fail if there's an errant newline somewhere around
+		// Reset inputPos (accepting the input, but the z-machine allows the game to unaccept it later on, so that's alright)
+		inputPos = [str length];
+		return res;
 	}
 }
 
 - (NSString*) backtrackInputOver: (NSString*) prefix {
 	if (prefix == nil) return nil;
-	
+		
 	if (inputLine != nil) {
 		// Input lines currently are unable to backtrack, so the prefix remains unaltered
 		return prefix;
@@ -2578,8 +2830,9 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 		int len = [prefix length];
 		if (len > [str length]) len = [str length];
 		
-		// Cut out a substring according to the length
-		str = [str substringWithRange: NSMakeRange([str length]-len, len)];
+		// Cut out a substring according to the length (and the current input position)
+		if (inputPos > [str length]) inputPos = [str length];
+		str = [str substringWithRange: NSMakeRange(inputPos-len, len)];
 		
 		// We compare lowercase versions of the string: this allows for things like Beyond Zork which write 'EXAMINE' but add 'examine' to the buffer
 		NSString* lowerPrefix = [prefix lowercaseString];
@@ -2708,6 +2961,10 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 		
 		if (nextInput == nil) {
 			// End of input
+			if (delegate && [delegate respondsToSelector: @selector(inputSourceHasFinished:)]) {
+				[delegate inputSourceHasFinished: inputSource];
+			}
+			
 			[inputSource release];
 			inputSource = nil;
 		} else {			
@@ -2725,6 +2982,10 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 		
 		if (nextInput == nil) {
 			// End of input
+			if (delegate && [delegate respondsToSelector: @selector(inputSourceHasFinished:)]) {
+				[delegate inputSourceHasFinished: inputSource];
+			}
+			
 			[inputSource release];
 			inputSource = nil;
 		} else {
@@ -2837,4 +3098,76 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 	}
 }
 
+- (ZoomTextToSpeech*) textToSpeech {
+	if (!textToSpeechReceiver) {
+		textToSpeechReceiver = [[ZoomTextToSpeech alloc] init];
+		[self addOutputReceiver: textToSpeechReceiver];
+	}
+	return textToSpeechReceiver;
+}
+
+// = Accessibility=
+
+- (NSString *)accessibilityActionDescription: (NSString*) action {
+	return [super accessibilityActionDescription:  action];
+}
+
+- (NSArray *)accessibilityActionNames {
+	return [super accessibilityActionNames];
+}
+
+- (BOOL)accessibilityIsAttributeSettable:(NSString *)attribute {
+	return [super accessibilityIsAttributeSettable: attribute];;
+}
+
+- (void)accessibilityPerformAction:(NSString *)action {
+	[super accessibilityPerformAction: action];
+}
+
+- (void)accessibilitySetValue: (id)value
+				 forAttribute: (NSString*) attribute {
+	// No settable attributes
+	[super accessibilitySetValue: value
+					forAttribute: attribute];
+}
+
+- (NSArray*) accessibilityAttributeNames {
+	NSMutableArray* result = [[super accessibilityAttributeNames] mutableCopy];
+	
+	[result addObjectsFromArray:[NSArray arrayWithObjects: 
+								 NSAccessibilityChildrenAttribute,
+								 nil]];
+	
+	return result;
+}
+
+- (id)accessibilityAttributeValue:(NSString *)attribute {
+	if ([attribute isEqualToString: NSAccessibilityChildrenAttribute]) {
+		return [NSArray arrayWithObjects: [textScroller upperWindowView], textView, nil];
+	} else if ([attribute isEqualToString: NSAccessibilityRoleDescriptionAttribute]) {
+		return [NSString stringWithFormat: @"Zoom view"];
+	} else if ([attribute isEqualToString: NSAccessibilityRoleAttribute]) {
+		return NSAccessibilityUnknownRole;
+	} else if ([attribute isEqualToString: NSAccessibilityFocusedAttribute]) {
+		NSView* viewResponder = (NSView*)[[self window] firstResponder];
+		if ([viewResponder isKindOfClass: [NSView class]]) {
+			while (viewResponder != nil) {
+				if (viewResponder == self) return [NSNumber numberWithBool: YES];
+				
+				viewResponder = [viewResponder superview];
+			}
+		}
+		
+		return [NSNumber numberWithBool: NO];
+	} else if ([attribute isEqualToString: NSAccessibilityParentAttribute]) {
+		//return nil;
+	}
+	
+	return [super accessibilityAttributeValue: attribute];
+}
+
+- (BOOL)accessibilityIsIgnored {
+	return YES;
+}
+		 
 @end
